#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Models for storing data from Campaign Disclosure Statements (Form 461) Part 5 schedules.
"""
from django.db import models
from calaccess_processed_filings.models.campaign import CampaignExpenditureItemBase


class Form461Part5Item(CampaignExpenditureItemBase):
    """
    Payments made by Form 461 filers.
    """
    filing = models.ForeignKey(
        'Form461Filing',
        related_name='part_5_items',
        null=True,
        on_delete=models.SET_NULL,
        help_text='Foreign key referring to the Form 461 on which the '
                  'payment was reported (from EXPN_CD.FILING_ID)',
    )

    class Meta:
        """
        Model options.
        """
        app_label = 'calaccess_processed_filings'
        unique_together = ((
            'filing',
            'line_item',
        ),)
        verbose_name = "Form 461 (Campaign Disclosure) Part 5 item"

    def __str__(self):
        return '%s-%s' % (self.filing, self.line_item)


class Form461Part5ItemVersion(CampaignExpenditureItemBase):
    """
    Every version of each payment made by a Form 461 filing.
    """
    filing_version = models.ForeignKey(
        'Form461FilingVersion',
        related_name='part_5_items',
        null=True,
        on_delete=models.SET_NULL,
        help_text='Foreign key referring to the version of the Form 461 that '
                  'includes the payment made'
    )

    class Meta:
        """
        Model options.
        """
        app_label = 'calaccess_processed_filings'
        unique_together = ((
            'filing_version',
            'line_item',
        ),)
        index_together = ((
            'filing_version',
            'line_item',
        ),)
        verbose_name = "Form 461 (Campaign Disclosure) Part 5 item version"

    def __str__(self):
        return '%s-%s-%s' % (
            self.filing_version.filing_id,
            self.filing_version.amend_id,
            self.line_item
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Proxy models for augmenting our source data tables with methods useful for processing.
"""
from __future__ import unicode_literals

# Models
from calaccess_processed.proxies import OCDProxyModelMixin
from opencivicdata.core.models import Jurisdiction

# Managers
from calaccess_processed_elections.managers import OCDJurisdictionManager


class OCDJurisdictionProxy(Jurisdiction, OCDProxyModelMixin):
    """
    A proxy on the OCD Jurisdiction model with helper methods.
    """
    objects = OCDJurisdictionManager()

    class Meta:
        """
        Make this a proxy model.
        """
        app_label = "calaccess_processed_elections"
        proxy = True



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Proxy models for augmenting our source data tables with methods useful for processing.
"""
from __future__ import unicode_literals
from django.apps import apps
from calaccess_processed.managers import BulkLoadSQLManager


class RawFilerToFilerTypeCdManager(BulkLoadSQLManager):
    """
    Custom helpers for the calaccess_raw FilerToFilerTypeCd model.
    """
    def get_office_by_filer_id_and_date(self, filer_id, election_date):
        """
        Lookup the office for the given filer_id, effective before election_date.

        Return a string containg the office name and district number (if applicable),
        or None if not found.
        """
        LookupCodesCd = apps.get_model("calaccess_raw", "LookupCodesCd")

        # Try a straight query for it
        try:
            ftft = self.get_queryset().filter(filer_id=filer_id, effect_dt__lte=election_date).latest('effect_dt')
        except (self.model.DoesNotExist, ValueError):
            # If you don't find it, quit.
            return None

        # Look up the race type
        try:
            office = LookupCodesCd.objects.get(code_id=ftft.race)
        except (LookupCodesCd.DoesNotExist, LookupCodesCd.MultipleObjectsReturned):
            # If you can't find it, quit.
            return None

        # If we don't have a valid district code, just return the name.
        if not ftft.district_cd or ftft.district_cd == 0:
            return "{}".format(office).strip()

        # Otherwise, get the district and tack that on
        try:
            district = LookupCodesCd.objects.get(code_id=ftft.district_cd)
        except (LookupCodesCd.DoesNotExist, LookupCodesCd.MultipleObjectsReturned):
            return None

        # If you found a district, return the string with office combined in there
        return '{} {}'.format(office, district).strip()



from django.db import ProgrammingError
from django.shortcuts import render
from collections import defaultdict
# from calaccess_processed_campaignfinance.proxies import (
#     OCDCommitteeProxy,
#     OCDFilingProxy,
#     OCDFilingActionProxy
# )
from calaccess_processed_elections.proxies import (
    OCDPostProxy,
    OCDPersonProxy,
    OCDElectionProxy,
    OCDCandidacyProxy,
    OCDPartyProxy
)
from opencivicdata.elections.models import CandidateContest
from django.views.generic import DetailView, ListView


def index(request):
    return render(request, "index.html")


def election_list(request):
    object_list = OCDElectionProxy.objects.all()
    group_dict = defaultdict()
    for obj in object_list:
        group_dict.setdefault(obj.date.year, []).append(obj)
    group_list = sorted(group_dict.items(), key=lambda x:x[0], reverse=True)
    context = dict(object_list=object_list, group_list=group_list)
    template = "election_list.html"
    return render(request, template, context)


def election_detail(request, id):
    obj = OCDElectionProxy.objects.get(id=id)
    context = dict(object=obj)
    template = "election_detail.html"
    return render(request, template, context)


class CandidateContestDetail(DetailView):
    model = CandidateContest
    template_name = 'candidatecontest_detail.html'

    def get_context_data(self, object=None):
        context = super(CandidateContestDetail, self).get_context_data()
        context['candidate_list'] = OCDCandidacyProxy.objects.filter(contest=context['object'])
        return context


class PostDetail(DetailView):
    model = OCDPostProxy
    template_name = "post_detail.html"


class PersonList(ListView):
    model = OCDPersonProxy
    template_name = "person_list.html"


class PersonDetail(DetailView):
    model = OCDPersonProxy
    template_name = "person_detail.html"

    def get_context_data(self, object=None):
        context = super(PersonDetail, self).get_context_data()
        context['candidate_list'] = OCDCandidacyProxy.objects.filter(person=context['object'])
        return context


class PartyList(ListView):
    model = OCDPartyProxy
    template_name = "party_list.html"


class PartyDetail(DetailView):
    model = OCDPartyProxy
    template_name = "party_detail.html"


class CandidateNoPartyList(ListView):
    """
    Lists all the OCD candidates with an unknown party.
    """
    template_name = "candidatenoparty_list.html"

    def get_queryset(self):
        try:
            return OCDCandidacyProxy.objects.filter(party__in=[
                OCDPartyProxy.objects.unknown()
            ])
        except (OCDPartyProxy.DoesNotExist, ProgrammingError):
            return OCDCandidacyProxy.objects.none()


class PrimaryNoPartyList(ListView):
    """
    Lists all the OCD partisan primaries with an unknown party.
    """
    template_name = "electionnoparty_list.html"

    def get_queryset(self):
        primary_list = OCDElectionProxy.partisan_primaries.all()
        contest_list = CandidateContest.objects.filter(election__in=primary_list)
        try:
            return contest_list.filter(party__in=[
                OCDPartyProxy.objects.unknown(),
                OCDPartyProxy.objects.get(name="NO PARTY PREFERENCE")
            ])
        except OCDPartyProxy.DoesNotExist:
            return CandidateContest.objects.none()


class CommitteeList(ListView):
    model = OCDCommitteeProxy
    template_name = "committee_list.html"


class CommitteeDetail(DetailView):
    model = OCDCommitteeProxy
    template_name = "committee_detail.html"


class FilingDetail(DetailView):
    model = OCDFilingProxy
    template_name = "filing_detail.html"


class FilingActionDetail(DetailView):
    model = OCDFilingActionProxy
    template_name = "filingaction_detail.html"



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Models for storing data from Schedule 497, the Late Contribution Reports.

More about the filing: http://calaccess.californiacivicdata.org/documentation/calaccess-forms/f497/
"""
from django.db import models
from calaccess_processed_filings.models.base import FilingBaseModel


class Form497ItemBase(FilingBaseModel):
    """
    Abstract base model for items reported on Schedule 497 filings.

    On Schedule 497, campaign filers are required to report late contributions
    received or made in the 90 days leading up to an election.
    """
    line_item = models.IntegerField(
        verbose_name='line item',
        db_index=True,
        null=False,
        help_text='Line number of the filing form where the transaction is '
                  'itemized (from S497_CD.LINE_ITEM)',
    )
    date_received = models.DateField(
        verbose_name='date received',
        db_index=True,
        null=True,
        help_text='Date the late contribution was received (from S497_CD.'
                  'CTRIB_DATE, unless NULL then from S497_CD.DATE_THRU)'
    )
    date_received_thru = models.DateField(
        verbose_name='date received thru',
        null=True,
        help_text='End date for late contributions received over a range of '
                  'days(from S497_CD.DATE_THRU)',
    )
    amount_received = models.DecimalField(
        verbose_name='amount received',
        decimal_places=2,
        max_digits=16,
        help_text='Dollar amount received (from S497_CD.AMOUNT)',
    )
    transaction_id = models.CharField(
        verbose_name='transaction id',
        max_length=20,
        db_index=True,
        help_text='Identifies a unique transaction across versions of the a '
                  'given Schedule 497 filing (from S497_CD.TRAN_ID)'
    )
    memo_reference_number = models.CharField(
        verbose_name='memo reference number',
        max_length=20,
        blank=True,
        help_text='Reference number for the memo attached to the transaction '
                  '(from S497_CD.MEMO_REFNO)',
    )

    class Meta:
        """
        Model options.
        """
        app_label = 'calaccess_processed_filings'
        abstract = True



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Proxy models for augmenting our source data tables with methods useful for processing.
"""
from __future__ import unicode_literals
from django.utils.text import get_text_list

# Models
from django.db.models import Count
from opencivicdata.elections.models import (
    Election,
    ElectionIdentifier,
    ElectionSource
)
from calaccess_processed.proxies import OCDProxyModelMixin
from ..core.posts import OCDPostProxy
from .candidatecontests import OCDCandidateContestProxy

# Managers
from calaccess_processed.managers import BulkLoadSQLManager
from calaccess_processed_elections.managers import (
    OCDPartisanPrimaryManager,
    OCDElectionManager
)


class OCDElectionProxy(Election, OCDProxyModelMixin):
    """
    A proxy on the OCD Election model.
    """
    objects = OCDElectionManager()
    partisan_primaries = OCDPartisanPrimaryManager()

    copy_to_fields = (
        ('id',),
        ('name',),
        ('date',),
        ('division_id',),
        ('administrative_organization_id',),
        ('created_at',),
        ('updated_at',),
        ('extras',),
        ('locked_fields',),
    )

    class Meta:
        """
        Make this a proxy model.
        """
        app_label = "calaccess_processed_elections"
        proxy = True

    def add_election_type(self, election_type):
        """
        Add election_type to 'calaccess_election_type' in extras field (if missing).
        """
        if 'calaccess_election_type' in self.extras.keys():
            # and if this one isn't included
            if election_type not in self.extras[
                'calaccess_election_type'
            ]:
                # then append
                self.extras['calaccess_election_type'].append(election_type)
                # and save
                self.save()
        else:
            # if election doesn't already have types, add the key
            self.extras['calaccess_election_type'] = [election_type]
            # and save
            self.save()

        return

    def add_election_id(self, election_id):
        """
        Add election_id to identifiers, if missing.
        """
        if not self.identifiers.filter(
            scheme='calaccess_election_id',
            identifier=election_id,
        ).exists():
            self.identifiers.create(
                scheme='calaccess_election_id',
                identifier=election_id,
            )
            self.save()

        return

    def get_regular_senate_contests_in_wrong_districts(self):
        """
        Get a list of regular senate contests in districts that shouldn't be contested.
        """
        if self.is_gubernatorial_election:
            # in gubernatorial elections,
            # odd-numbered senate districts should not contested
            contests = [
                c for c in self.senate_contests.regular()
                if int(c.division.subid2) % 2 != 0
            ]
        else:
            # in non-gubernatorial elections,
            # even-numbered senate districts should not be contests
            contests = [
                c for c in self.senate_contests.regular()
                if int(c.division.subid2) % 2 == 0
            ]
        return contests

    @property
    def assembly_contests(self):
        """
        State Assembly CandidateContests occurring in the election.
        """
        return self.candidate_contest_proxies.assembly()

    @property
    def candidate_contest_proxies(self):
        """
        A QuerySet of OCDCandidateContestProxy for the election.
        """
        return OCDCandidateContestProxy.objects.filter(election=self)

    @property
    def election_type(self):
        """
        Returns the primary CAL-ACCESS election type included with this record.
        """
        for et in self.extras.get('calaccess_election_type', []):
            if et in self.name:
                return et

    @property
    def election_types(self):
        """
        Returns all the CAL-ACCESS election types included with this record.
        """
        return self.extras.get('calaccess_election_type', [])

    @property
    def executive_contests(self):
        """
        State Executive Branch CandidateContests occurring in the election.
        """
        return self.candidate_contest_proxies.executive()

    @property
    def has_special_contests(self):
        """
        This election includes contests outside the regular election calendar.
        """
        special_election_types = set(
            ("SPECIAL ELECTION", "SPECIAL RUNOFF", "RECALL")
        )
        return len(
            special_election_types.intersection(self.election_types)
        ) > 0

    @property
    def identifier_list(self):
        """
        Returns a prettified list of OCD identifiers.
        """
        template = "{0.scheme}: {0.identifier}"
        return get_text_list([template.format(i) for i in self.identifiers.all()])

    @property
    def is_gubernatorial_election(self):
        """
        This election should include contests for Governor other executive branch offices.
        """
        # Governors are elected every four years, and the earliest such election
        # in CAL-ACCESS was 2002
        return (self.date.year - 2002) % 4 == 0

    @property
    def is_partisan_primary(self):
        """
        Returns whether or not this was a primary election held in the partisan era prior to 2012.
        """
        if 'PRIMARY' in self.election_types:
            if self.date.year < 2012:
                return True
        return False

    @property
    def regular_assembly_contest_count_actual(self):
        """
        Actual count of regular State Assembly contests.
        """
        return self.assembly_contests.regular().count()

    @property
    def regular_assembly_contest_count_expected(self):
        """
        Expected count of regular State Assembly contests (based on year).
        """
        assembly_office_count = OCDPostProxy.assembly.count()

        if "GENERAL" in self.election_types:
            expected_contest_count = assembly_office_count
        elif "PRIMARY" in self.election_types:
            if self.is_partisan_primary:
                # should be one contest for every distinct party in each
                # of the (80) assembly seats
                expected_contest_count = 0
                contests_q = self.assembly_contests.regular()
                contest_counts_by_party = contests_q.order_by().values(
                    'candidacies__party__name'
                ).annotate(
                    contest_count=Count('candidacies__contest', distinct=True)
                )
                for party in contest_counts_by_party:
                    expected_contest_count += party['contest_count']
            else:
                expected_contest_count = assembly_office_count
        else:
            expected_contest_count = 0

        return expected_contest_count

    @property
    def regular_executive_contest_count_actual(self):
        """
        Actual count of regular State Executive Branch contests.
        """
        return self.executive_contests.regular().count()

    @property
    def regular_executive_contest_count_expected(self):
        """
        Expected count of regular State Assembly contests (based on year).
        """
        exec_office_count = OCDPostProxy.executive.count()

        if self.is_gubernatorial_election:
            if "GENERAL" in self.election_types:
                expected_contest_count = exec_office_count
            elif "PRIMARY" in self.election_types:
                if self.is_partisan_primary:
                    # should be 1 contest for every distinct party in each
                    # of the executive branch offices (12)
                    expected_contest_count = 0
                    contests_q = self.executive_contests.regular()
                    contest_counts_by_party = contests_q.order_by().values(
                        'candidacies__party__name'
                    ).annotate(
                        contest_count=Count('candidacies__contest', distinct=True)
                    )
                    for party in contest_counts_by_party:
                        expected_contest_count += party['contest_count']
                else:
                    expected_contest_count = exec_office_count
            else:
                expected_contest_count = 0
        else:
            expected_contest_count = 0

        return expected_contest_count

    @property
    def regular_senate_contest_count_actual(self):
        """
        Actual count of regular State Senate contests.
        """
        return self.senate_contests.regular().count()

    @property
    def regular_senate_contest_count_expected(self):
        """
        Confirm the Election has the correct count of regular Assembly contests.
        """
        # half of the senates are filled every two years
        senate_office_count = int(OCDPostProxy.senate.count() / 2)

        if "GENERAL" in self.election_types:
            expected_contest_count = senate_office_count
        elif "PRIMARY" in self.election_types:
            if self.is_partisan_primary:
                # should be one contest for every distinct party in every
                # other senate district (20)
                expected_contest_count = 0
                contests_q = self.senate_contests.regular()
                contest_counts_by_party = contests_q.order_by().values(
                    'candidacies__party__name'
                ).annotate(
                    contest_count=Count('candidacies__contest', distinct=True)
                )
                for party in contest_counts_by_party:
                    expected_contest_count += party['contest_count']
            else:
                expected_contest_count = senate_office_count
        else:
            expected_contest_count = 0

        return expected_contest_count

    @property
    def senate_contests(self):
        """
        State Senate CandidateContests occurring in the election.
        """
        return self.candidate_contest_proxies.senate()

    @property
    def source_list(self):
        """
        Returns a prettified list of OCD sources.
        """
        return get_text_list(list(self.sources.all()))


class OCDElectionIdentifierProxy(ElectionIdentifier, OCDProxyModelMixin):
    """
    A proxy on the OCD ElectionIdentifier model.
    """
    objects = BulkLoadSQLManager()

    class Meta:
        """
        Make this a proxy model.
        """
        app_label = "calaccess_processed_elections"
        proxy = True


class OCDElectionSourceProxy(ElectionSource, OCDProxyModelMixin):
    """
    A proxy on the OCD ElectionSource model.
    """
    objects = BulkLoadSQLManager()

    class Meta:
        """
        Make this a proxy model.
        """
        app_label = "calaccess_processed_elections"
        proxy = True



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Unittests for management commands.
"""
from datetime import date
from unittest import TestCase
from calaccess_processed_elections import get_expected_election_date


class ElectionDatesTest(TestCase):
    """
    Test election date parsing utilities.
    """
    def test_2016_primary_date(self):
        """
        Confirm correct calculation of 2016 primary date.
        """
        self.assertEqual(
            date(2016, 6, 7),
            get_expected_election_date(2016, 'PRIMARY'),
        )

    def test_2016_general_date(self):
        """
        Confirm correct calculation of 2016 general date.
        """
        self.assertEqual(
            date(2016, 11, 8),
            get_expected_election_date(2016, 'GENERAL'),
        )

    def test_2014_primary_date(self):
        """
        Confirm correct calculation of 2014 primary date.
        """
        self.assertEqual(
            date(2014, 6, 3),
            get_expected_election_date(2014, 'PRIMARY'),
        )

    def test_2010_general_date(self):
        """
        Confirm correct calculation of 2010 general date.
        """
        self.assertEqual(
            date(2010, 11, 2),
            get_expected_election_date(2010, 'GENERAL'),
        )


class ScrapedCandidatElectioneNameParsingTest(TestCase):
    """
    Test how candidate names are parsed from scraped data.
    """
    def test_parse_of_general_election_name(self):
        """
        Test .parse_election_name() on "2016 GENERAL".
        """
        from calaccess_processed_elections.proxies import ScrapedCandidateElectionProxy
        parsed_name = ScrapedCandidateElectionProxy(name='2016 GENERAL').parsed_name
        assert parsed_name == {
            'year': 2016,
            'type': 'GENERAL',
            'office': None,
            'district': None,
        }

    def test_parse_of_state_senate_special_election_name(self):
        """
        Test .parse_election_name() on "2009 SPECIAL ELECTION (STATE SENATE 26)".
        """
        from calaccess_processed_elections.proxies import ScrapedCandidateElectionProxy
        parsed_name = ScrapedCandidateElectionProxy(name='2009 SPECIAL ELECTION (STATE SENATE 26)').parsed_name
        assert parsed_name == {
            'year': 2009,
            'type': 'SPECIAL ELECTION',
            'office': 'STATE SENATE',
            'district': 26,
        }

    def test_parse_of_assembly_special_runoff_name(self):
        """
        Test .parse_election_name() on "2010 SPECIAL RUNOFF (ASSEMBLY 72)".
        """
        from calaccess_processed_elections.proxies import ScrapedCandidateElectionProxy
        parsed_name = ScrapedCandidateElectionProxy(name='2010 SPECIAL RUNOFF (ASSEMBLY 72)').parsed_name
        assert parsed_name == {
            'year': 2010,
            'type': 'SPECIAL RUNOFF',
            'office': 'ASSEMBLY',
            'district': 72,
        }

    def test_parse_of_special_election_name_without_district(self):
        """
        Test .parse_election_name() on "2003 SPECIAL ELECTION (GOVERNOR)".
        """
        from calaccess_processed_elections.proxies import ScrapedCandidateElectionProxy
        parsed_name = ScrapedCandidateElectionProxy(name='2003 SPECIAL ELECTION (GOVERNOR)').parsed_name
        assert parsed_name == {
            'year': 2003,
            'type': 'SPECIAL ELECTION',
            'office': 'GOVERNOR',
            'district': None,
        }



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Custom administration panels for campaign models.
"""
from __future__ import unicode_literals
from django.contrib import admin
from calaccess_processed_filings import models
from calaccess_raw.admin.base import BaseAdmin


@admin.register(models.Form460Filing)
class Form460FilingAdmin(BaseAdmin):
    """
    Custom admin for the Form460Filing model.
    """
    list_display = (
        'filing_id',
        'amendment_count',
        'filer_id',
        'filer_lastname',
        'filer_firstname',
        'date_filed',
        'election_date',
        'total_contributions',
        'total_expenditures_made',
        'ending_cash_balance',
    )


@admin.register(models.Form460FilingVersion)
class Form460FilingVersionAdmin(BaseAdmin):
    """
    Custom admin for the Form460FilingVersion model.
    """
    list_display = (
        'filing',
        'amend_id',
        'filer_id',
        'filer_lastname',
        'filer_firstname',
        'date_filed',
        'election_date',
        'total_contributions',
        'total_expenditures_made',
        'ending_cash_balance',
    )


@admin.register(models.Form460ScheduleASummary)
class Form460ScheduleASummaryAdmin(BaseAdmin):
    """
    Custom admin for the Form460ScheduleASummary model.
    """
    list_display = (
        'filing',
        'itemized_contributions',
        'unitemized_contributions',
        'total_contributions'
    )
    search_fields = (
        'filing__filing_id',
    )


@admin.register(models.Form460ScheduleASummaryVersion)
class Form460ScheduleASummaryVersionAdmin(BaseAdmin):
    """
    Custom admin for the Form460ScheduleASummaryVersion model.
    """
    list_display = (
        'filing_version',
        'itemized_contributions',
        'unitemized_contributions',
        'total_contributions'
    )


@admin.register(models.Form460ScheduleAItem)
class Form460ScheduleAItemAdmin(BaseAdmin):
    """
    Custom admin for the Form460ScheduleAItem model.
    """
    list_display = (
        'filing',
        'line_item',
        'date_received',
        'amount',
        'transaction_id',
        'contributor_code',
        'contributor_lastname',
        'contributor_firstname',
    )


@admin.register(models.Form460ScheduleAItemVersion)
class Form460ScheduleAItemVersionAdmin(BaseAdmin):
    """
    Custom admin for the Form460ScheduleAItemVersion model.
    """
    list_display = (
        'filing_version',
        'line_item',
        'date_received',
        'amount',
        'transaction_id',
        'contributor_code',
        'contributor_lastname',
        'contributor_firstname',
    )


@admin.register(models.Form460ScheduleB1Item)
class Form460ScheduleB1ItemAdmin(BaseAdmin):
    """
    Custom admin for the Form460ScheduleB1Item model.
    """
    list_display = (
        'filing',
        'line_item',
        'lender_code',
        'lender_lastname',
        'begin_period_balance',
        'amount_received',
        'amount_paid',
        'amount_forgiven',
        'end_period_balance',
        'date_due',
        'interest_paid',
        'interest_rate',
        'original_amount',
        'date_incurred',
        'cumulative_ytd_contributions',
        'transaction_id',
    )


@admin.register(models.Form460ScheduleB1ItemVersion)
class Form460ScheduleB1ItemVersionAdmin(BaseAdmin):
    """
    Custom admin for the Form460ScheduleB1ItemVersion model.
    """
    list_display = (
        'filing_version',
        'line_item',
        'lender_code',
        'lender_lastname',
        'begin_period_balance',
        'amount_received',
        'amount_paid',
        'amount_forgiven',
        'end_period_balance',
        'date_due',
        'interest_paid',
        'interest_rate',
        'original_amount',
        'date_incurred',
        'cumulative_ytd_contributions',
        'transaction_id',
    )


@admin.register(models.Form460ScheduleB2Item)
class Form460ScheduleB2ItemAdmin(BaseAdmin):
    """
    Custom admin for the Form460ScheduleB2Item model.
    """
    list_display = (
        'filing',
        'line_item',
        'guarantor_code',
        'guarantor_lastname',
        'amount_guaranteed_this_period',
        'balance_outstanding_to_date',
        'cumulative_ytd_amount',
        'loan_date',
        'transaction_id',
    )


@admin.register(models.Form460ScheduleB2ItemVersion)
class Form460ScheduleB2ItemVersionAdmin(BaseAdmin):
    """
    Custom admin for the Form460ScheduleB2ItemVersion model.
    """
    list_display = (
        'filing_version',
        'line_item',
        'guarantor_code',
        'guarantor_lastname',
        'amount_guaranteed_this_period',
        'balance_outstanding_to_date',
        'cumulative_ytd_amount',
        'loan_date',
        'transaction_id',
    )


@admin.register(models.Form460ScheduleB2ItemOld)
class Form460ScheduleB2ItemOldAdmin(BaseAdmin):
    """
    Custom admin for the Form460ScheduleB2ItemOld model.
    """
    list_display = (
        'filing',
        'line_item',
        'lender_code',
        'lender_lastname',
        'date_repaid_or_forgiven',
        'date_of_original_loan',
        'interest_rate',
        'repayment_type',
        'amount_repaid_or_forgiven',
        'outstanding_principle',
        'interest_paid',
        'transaction_id',
    )


@admin.register(models.Form460ScheduleB2ItemVersionOld)
class Form460ScheduleB2ItemVersionOldAdmin(BaseAdmin):
    """
    Custom admin for the Form460ScheduleB2ItemVersionOld model.
    """
    list_display = (
        'filing_version',
        'line_item',
        'lender_code',
        'lender_lastname',
        'date_repaid_or_forgiven',
        'date_of_original_loan',
        'interest_rate',
        'repayment_type',
        'amount_repaid_or_forgiven',
        'outstanding_principle',
        'interest_paid',
        'transaction_id',
    )


@admin.register(models.Form460ScheduleCSummary)
class Form460ScheduleCSummaryAdmin(BaseAdmin):
    """
    Custom admin for the Form460ScheduleCSummary model.
    """
    list_display = (
        'filing',
        'itemized_contributions',
        'unitemized_contributions',
        'total_contributions'
    )
    search_fields = (
        'filing__filing_id',
    )


@admin.register(models.Form460ScheduleCSummaryVersion)
class Form460ScheduleCSummaryVersionAdmin(BaseAdmin):
    """
    Custom admin for the Form460ScheduleCSummaryVersion model.
    """
    list_display = (
        'filing_version',
        'itemized_contributions',
        'unitemized_contributions',
        'total_contributions'
    )


@admin.register(models.Form460ScheduleCItem)
class Form460ScheduleCItemAdmin(BaseAdmin):
    """
    Custom admin for the Form460ScheduleCItem model.
    """
    list_display = (
        'filing',
        'line_item',
        'date_received',
        'fair_market_value',
        'transaction_id',
        'contributor_code',
        'contributor_lastname',
        'contributor_firstname',
    )


@admin.register(models.Form460ScheduleCItemVersion)
class Form460ScheduleCItemVersionAdmin(BaseAdmin):
    """
    Custom admin for the Form460ScheduleCItemVersion model.
    """
    list_display = (
        'filing_version',
        'line_item',
        'date_received',
        'fair_market_value',
        'transaction_id',
        'contributor_code',
        'contributor_lastname',
        'contributor_firstname',
    )


@admin.register(models.Form460ScheduleDItem)
class Form460ScheduleDItemAdmin(BaseAdmin):
    """
    Custom admin for the Form460ScheduleDItem model.
    """
    list_display = (
        'filing',
        'line_item',
        'expense_date',
        'payee_code',
        'payee_lastname',
        'payee_firstname',
        'support_oppose_code',
        'ballot_measure_name',
        'candidate_lastname',
        'amount',
        'cumulative_election_amount',
        'cumulative_ytd_amount',
        'transaction_id',
    )


@admin.register(models.Form460ScheduleDItemVersion)
class Form460ScheduleDItemVersionAdmin(BaseAdmin):
    """
    Custom admin for the Form460ScheduleDItemVersion model.
    """
    list_display = (
        'filing_version',
        'line_item',
        'expense_date',
        'payee_code',
        'payee_lastname',
        'payee_firstname',
        'support_oppose_code',
        'ballot_measure_name',
        'candidate_lastname',
        'amount',
        'cumulative_election_amount',
        'cumulative_ytd_amount',
        'transaction_id',
    )


@admin.register(models.Form460ScheduleEItem)
class Form460ScheduleEItemAdmin(BaseAdmin):
    """
    Custom admin for the Form460ScheduleEItem model.
    """
    list_display = (
        'filing',
        'line_item',
        'expense_date',
        'payee_code',
        'payee_lastname',
        'payee_firstname',
        'amount',
        'cumulative_ytd_amount',
        'transaction_id',
    )


@admin.register(models.Form460ScheduleEItemVersion)
class Form460ScheduleEItemVersionAdmin(BaseAdmin):
    """
    Custom admin for the Form460ScheduleEItemVersion model.
    """
    list_display = (
        'filing_version',
        'line_item',
        'expense_date',
        'payee_code',
        'payee_lastname',
        'payee_firstname',
        'amount',
        'cumulative_ytd_amount',
        'transaction_id',
    )


@admin.register(models.Form460ScheduleESubItem)
class Form460ScheduleESubItemAdmin(BaseAdmin):
    """
    Custom admin for the Form460ScheduleESubItem model.
    """
    list_display = (
        'filing',
        'line_item',
        'expense_date',
        'payee_code',
        'payee_lastname',
        'payee_firstname',
        'amount',
        'cumulative_ytd_amount',
        'transaction_id',
        'parent_transaction_id',
        'memo_reference_number',
    )


@admin.register(models.Form460ScheduleESubItemVersion)
class Form460ScheduleESubItemVersionAdmin(BaseAdmin):
    """
    Custom admin for the Form460ScheduleESubItemVersion model.
    """
    list_display = (
        'filing_version',
        'line_item',
        'expense_date',
        'payee_code',
        'payee_lastname',
        'payee_firstname',
        'amount',
        'cumulative_ytd_amount',
        'transaction_id',
        'parent_transaction_id',
        'memo_reference_number',
    )


@admin.register(models.Form460ScheduleFItem)
class Form460ScheduleFItemAdmin(BaseAdmin):
    """
    Custom admin for the Form460ScheduleFItem model.
    """
    list_display = (
        'filing',
        'line_item',
        'payee_code',
        'payee_lastname',
        'payee_firstname',
        'begin_balance',
        'amount_paid',
        'amount_incurred',
        'end_balance',
        'transaction_id',
        'parent_transaction_id',
        'memo_reference_number',
    )


@admin.register(models.Form460ScheduleFItemVersion)
class Form460ScheduleFItemVersionAdmin(BaseAdmin):
    """
    Custom admin for the Form460ScheduleFItemVersion model.
    """
    list_display = (
        'filing_version',
        'line_item',
        'payee_code',
        'payee_lastname',
        'payee_firstname',
        'begin_balance',
        'amount_paid',
        'amount_incurred',
        'end_balance',
        'transaction_id',
        'parent_transaction_id',
        'memo_reference_number',
    )


@admin.register(models.Form460ScheduleGItem)
class Form460ScheduleGItemAdmin(BaseAdmin):
    """
    Custom admin for the Form460ScheduleGItem model.
    """
    list_display = (
        'filing',
        'line_item',
        'agent_lastname',
        'agent_firstname',
        'expense_date',
        'payee_code',
        'payee_lastname',
        'payee_firstname',
        'amount',
        'cumulative_ytd_amount',
        'transaction_id',
        'parent_transaction_id',
        'parent_schedule',
        'memo_reference_number',
    )


@admin.register(models.Form460ScheduleGItemVersion)
class Form460ScheduleGItemVersionAdmin(BaseAdmin):
    """
    Custom admin for the Form460ScheduleGItemVersion model.
    """
    list_display = (
        'filing_version',
        'line_item',
        'agent_lastname',
        'agent_firstname',
        'expense_date',
        'payee_code',
        'payee_lastname',
        'payee_firstname',
        'amount',
        'cumulative_ytd_amount',
        'transaction_id',
        'parent_transaction_id',
        'parent_schedule',
        'memo_reference_number',
    )


@admin.register(models.Form460ScheduleHItem)
class Form460ScheduleHItemAdmin(BaseAdmin):
    """
    Custom admin for the Form460ScheduleHItem model.
    """
    list_display = (
        'filing',
        'line_item',
        'recipient_code',
        'recipient_lastname',
        'begin_period_balance',
        'amount_loaned',
        'amount_paid',
        'amount_forgiven',
        'end_period_balance',
        'date_due',
        'interest_received',
        'interest_rate',
        'original_amount',
        'date_incurred',
        'cumulative_ytd_contributions',
        'transaction_id',
    )


@admin.register(models.Form460ScheduleHItemVersion)
class Form460ScheduleHItemVersionAdmin(BaseAdmin):
    """
    Custom admin for the Form460ScheduleHItemVersion model.
    """
    list_display = (
        'filing_version',
        'line_item',
        'recipient_code',
        'recipient_lastname',
        'begin_period_balance',
        'amount_loaned',
        'amount_paid',
        'amount_forgiven',
        'end_period_balance',
        'date_due',
        'interest_received',
        'interest_rate',
        'original_amount',
        'date_incurred',
        'cumulative_ytd_contributions',
        'transaction_id',
    )


@admin.register(models.Form460ScheduleH2ItemOld)
class Form460ScheduleH2ItemOldAdmin(BaseAdmin):
    """
    Custom admin for the Form460ScheduleH2ItemOld model.
    """
    list_display = (
        'filing',
        'line_item',
        'recipient_code',
        'recipient_lastname',
        'date_repaid_or_forgiven',
        'date_of_original_loan',
        'interest_rate',
        'repayment_type',
        'amount_repaid_or_forgiven',
        'outstanding_principle',
        'transaction_id',
    )


@admin.register(models.Form460ScheduleH2ItemVersionOld)
class Form460ScheduleH2ItemVersionOldAdmin(BaseAdmin):
    """
    Custom admin for the Form460ScheduleH2ItemVersionOld model.
    """
    list_display = (
        'filing_version',
        'line_item',
        'recipient_code',
        'recipient_lastname',
        'date_repaid_or_forgiven',
        'date_of_original_loan',
        'interest_rate',
        'repayment_type',
        'amount_repaid_or_forgiven',
        'outstanding_principle',
        'transaction_id',
    )


@admin.register(models.Form460ScheduleIItem)
class Form460ScheduleIItemAdmin(BaseAdmin):
    """
    Custom admin for the Form460ScheduleIItem model.
    """
    list_display = (
        'filing',
        'line_item',
        'date_received',
        'amount',
        'receipt_description',
        'transaction_id',
        'contributor_code',
        'contributor_lastname',
        'contributor_firstname',
    )


@admin.register(models.Form460ScheduleIItemVersion)
class Form460ScheduleIItemVersionAdmin(BaseAdmin):
    """
    Custom admin for the Form460ScheduleIItemVersion model.
    """
    list_display = (
        'filing_version',
        'line_item',
        'date_received',
        'amount',
        'receipt_description',
        'transaction_id',
        'contributor_code',
        'contributor_lastname',
        'contributor_firstname',
    )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Models for storing Part 3 data from Schedule 496, the Late Independent Expenditure Reports.

More about the filing: https://calaccess.californiacivicdata.org/documentation/calaccess-forms/f496/
"""
from django.db import models
from calaccess_processed_filings.models.campaign import CampaignContributionBase


class Form496Part3ItemBase(CampaignContributionBase):
    """
    Abstract base model for items reported on Schedule A of Form 460 filings.

    On Schedule A, campaign filers are required to itemize monetary
    contributions received during the period covered by the filing.
    """
    amount = models.DecimalField(
        verbose_name='amount',
        decimal_places=2,
        max_digits=14,
        help_text="Amount received from the contributor in the period covered "
                  "by the filing (from RCPT_CD.AMOUNT)"
    )
    interest_rate = models.CharField(
        verbose_name='interest rate',
        max_length=30,
        blank=True,
        help_text='Interest rate of a loan. This is sometimes expressed as a '
                  'decimal (e.g., 0.10) and other times as a percent (e.g., '
                  '10.0% (from RCPT_CD.INT_RATE)'
    )

    class Meta:
        """
        Model options.
        """
        app_label = 'calaccess_processed_filings'
        abstract = True


class Form496Part3Item(Form496Part3ItemBase):
    """
    Monetary contributions of greater than $100 from the Form 496's Part 3.
    """
    filing = models.ForeignKey(
        'Form496Filing',
        related_name='part3_items',
        null=True,
        on_delete=models.SET_NULL,
        help_text='Foreign key referring to the Form 496 on which the monetary'
                  ' contribution was reported (from RCPT_CD.FILING_ID)',
    )

    class Meta:
        """
        Model options.
        """
        app_label = 'calaccess_processed_filings'
        unique_together = ((
            'filing',
            'line_item',
        ),)
        verbose_name = "Form 496 (Late Independent Expenditure) Part 3 item"

    def __str__(self):
        return '%s-%s' % (self.filing, self.line_item)


class Form496Part3ItemVersion(Form496Part3ItemBase):
    """
    Every version of monetary contributions of greater than $100 from the Form 496's Part 3.
    """
    filing_version = models.ForeignKey(
        'Form496FilingVersion',
        related_name='part3_items',
        null=True,
        on_delete=models.SET_NULL,
        help_text='Foreign key referring to the version of the Form 496 that includes the received contribution'
    )

    class Meta:
        """
        Model options.
        """
        app_label = 'calaccess_processed_filings'
        unique_together = ((
            'filing_version',
            'line_item',
        ),)
        index_together = ((
            'filing_version',
            'line_item',
        ),)
        verbose_name = "Form 496 (Late Independent Expenditure) Part 3 item version"

    def __str__(self):
        return '%s-%s-%s' % (
            self.filing_version.filing_id,
            self.filing_version.amend_id,
            self.line_item
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Models for tracking processing of CAL-ACCESS snapshots over time.
"""
from django.apps import apps
from hurry.filesize import size as sizeformat

# Models
from django.db import models
from ia_storage.fields import InternetArchiveFileField


class ProcessedDataVersion(models.Model):
    """
    A version of CAL-ACCESS processed data.
    """
    raw_version = models.OneToOneField(
        'calaccess_raw.RawDataVersion',
        related_name='processed_version',
        verbose_name='raw data version',
        help_text='Foreign key referencing the raw data version processed',
        on_delete=models.CASCADE
    )
    process_start_datetime = models.DateTimeField(
        null=True,
        verbose_name='date and time processing started',
        help_text='Date and time when the processing of the CAL-ACCESS version'
                  ' started',
    )
    process_finish_datetime = models.DateTimeField(
        null=True,
        verbose_name='date and time update finished',
        help_text='Date and time when the processing of the CAL-ACCESS version'
                  ' finished',
    )

    class Meta:
        """
        Meta model options.
        """
        app_label = 'calaccess_processed'
        verbose_name = 'TRACKING: CAL-ACCESS processed data version'
        ordering = ('-process_start_datetime',)
        get_latest_by = 'process_start_datetime'

    def __str__(self):
        return str(self.raw_version.release_datetime)

    @property
    def update_completed(self):
        """
        Check if the database update to the version completed.

        Return True or False.
        """
        if self.process_finish_datetime:
            is_completed = True
        else:
            is_completed = False

        return is_completed

    @property
    def update_stalled(self):
        """
        Check if the database update to the version started but did not complete.

        Return True or False.
        """
        if self.process_start_datetime and not self.update_finish_datetime:
            is_stalled = True
        else:
            is_stalled = False

        return is_stalled

    def check_processed_model(self, model):
        """
        Return True if model was processed in this version.
        """
        try:
            file_name = model().file_name
        except AttributeError:
            file_name = model().klass_name
        return self.files.filter(file_name=file_name).exists()


class ProcessedDataZip(models.Model):
    """
    A zip file containing a subset of processed data files for a version.
    """
    version = models.ForeignKey(
        'ProcessedDataVersion',
        on_delete=models.CASCADE,
        related_name='zips',
        verbose_name='processed data version',
        help_text='Foreign key referencing the processed version of CAL-ACCESS'
    )
    zip_archive = InternetArchiveFileField(
        max_length=255,
        verbose_name='zip archive',
        help_text='An archived zip of processed files'
    )
    zip_size = models.BigIntegerField(
        default=0,
        verbose_name='size of zip (in bytes)',
        help_text='The expected size (in bytes) of the zip '
    )
    created_datetime = models.DateTimeField(
        auto_now_add=True,
        verbose_name='date and time zip created',
        help_text='Date and time when zip was created',
    )

    class Meta:
        """
        Meta model options.
        """
        app_label = 'calaccess_processed'
        verbose_name = 'TRACKING: CAL-ACCESS processed data zip'
        ordering = ('-version', 'zip_archive')
        unique_together = ('version', 'zip_archive')

    def __str__(self):
        return '{0} ({1})'.format(self.zip_archive, self.version)

    def pretty_zip_size(self):
        """
        Returns a prettified version (e.g., "725M") of the zip's size.
        """
        if not self.zip_size:
            return None
        return sizeformat(self.zip_size)
    pretty_zip_size.short_description = 'processed zip size'
    pretty_zip_size.admin_order_field = 'processed zip size'


class ProcessedDataFile(models.Model):
    """
    A data file included in a processed version of CAL-ACCESS.
    """
    version = models.ForeignKey(
        'ProcessedDataVersion',
        on_delete=models.CASCADE,
        related_name='files',
        verbose_name='processed data version',
        help_text='Foreign key referencing the processed version of CAL-ACCESS'
    )
    file_name = models.CharField(
        max_length=100,
        verbose_name='processed data file name',
        help_text='Name of the processed data file without extension',
    )
    process_start_datetime = models.DateTimeField(
        null=True,
        verbose_name='date and time processing started',
        help_text='Date and time when the processing of the file started',
    )
    process_finish_datetime = models.DateTimeField(
        null=True,
        verbose_name='date and time processing finished',
        help_text='Date and time when the processing of the file finished',
    )
    records_count = models.IntegerField(
        null=False,
        default=0,
        verbose_name='records count',
        help_text='Count of records in the processed file'
    )
    file_archive = InternetArchiveFileField(
        blank=True,
        max_length=255,
        verbose_name='archive of processed file',
        help_text='An archive of the processed file'
    )
    file_size = models.BigIntegerField(
        null=False,
        default=0,
        verbose_name='size of processed data file (in bytes)',
        help_text='Size of the processed file (in bytes)'
    )

    class Meta:
        """
        Meta model options.
        """
        app_label = 'calaccess_processed'
        unique_together = (('version', 'file_name'),)
        verbose_name = 'TRACKING: CAL-ACCESS processed data file'
        ordering = ('-version_id', 'file_name',)

    def __str__(self):
        return self.file_name

    def pretty_file_size(self):
        """
        Returns a prettified version (e.g., "725M") of the processed file's size.
        """
        return sizeformat(self.file_size)
    pretty_file_size.short_description = 'processed file size'
    pretty_file_size.admin_order_field = 'processed file size'

    @property
    def is_flat(self):
        """
        Return True if the proxy model is used to flatten relational data models.
        """
        try:
            is_flat = self.model().is_flat
        except (AttributeError, TypeError):
            is_flat = False
        return is_flat

    @property
    def model(self):
        """
        Returns the model associated with this record.
        """
        lookup = apps.get_app_config("calaccess_processed").get_processed_file_lookup()
        return lookup[self.file_name]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Export and archive a .csv file for a given model.
"""
import os
import logging
from calaccess_raw import get_data_directory
from calaccess_processed.management.commands import CalAccessCommand
from calaccess_processed.models.tracking import ProcessedDataVersion, ProcessedDataFile

logger = logging.getLogger(__name__)


class Command(CalAccessCommand):
    """
    Export and archive a .csv file for a given model.
    """
    help = 'Export and archive a .csv file for a given model.'

    def add_arguments(self, parser):
        """
        Adds custom arguments specific to this command.
        """
        super(Command, self).add_arguments(parser)
        parser.add_argument('model_name', help="Name of the model to archive")

    def get_model(self, processed_file):
        """
        Get the model linked to this processed file record.
        """
        raise NotImplementedError

    def handle(self, *args, **options):
        """
        Make it happen.
        """
        super(Command, self).handle(*args, **options)

        # Parse model name
        self.model_name = options['model_name']

        # Log out what we're doing ...
        self.log(" Archiving %s.csv" % self.model_name)

        # ... get the current version ...
        version = ProcessedDataVersion.objects.latest('process_start_datetime')

        # ... and the processed file object ...
        try:
            processed_file = version.files.get(file_name=self.model_name)
        except ProcessedDataFile.DoesNotExist:
            processed_file = version.files.create(file_name=self.model_name)

        # Get the data obj that is paired with the processed_file obj
        data_model = self.get_model(processed_file)

        # Update the records count
        processed_file.records_count = data_model.objects.count()

        # Save it
        processed_file.save()

        # Figure out the path where we will save the file
        csv_dir = os.path.join(
            get_data_directory(),
            'processed',
            data_model().klass_group.lower()
        )
        os.path.exists(csv_dir) or os.mkdir(csv_dir)
        csv_name = '{}.csv'.format(processed_file.file_name)
        csv_path = os.path.join(csv_dir, csv_name)

        # Export a new one
        try:
            copy_to_fields = tuple(i[0] for i in data_model.copy_to_fields)
        except AttributeError:
            copy_to_fields = tuple()
        data_model.objects.to_csv(csv_path, *copy_to_fields)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Admins for Schedule 497, the late contribution report.
"""
from __future__ import unicode_literals
from django.contrib import admin
from calaccess_processed_filings import models
from calaccess_raw.admin.base import BaseAdmin


@admin.register(models.Form496Filing)
class Form496FilingAdmin(BaseAdmin):
    """
    Custom admin for the Form496Filing model.
    """
    list_display = (
        'filing_id',
        'amendment_count',
        'filer_id',
        'filer_lastname',
        'filer_firstname',
        'date_filed',
        'election_date',
    )


@admin.register(models.Form496FilingVersion)
class Form496FilingVersionAdmin(BaseAdmin):
    """
    Custom admin for the Form496FilingVersion model.
    """
    list_display = (
        'filing',
        'amend_id',
        'filer_id',
        'filer_lastname',
        'filer_firstname',
        'date_filed',
        'election_date',
    )


@admin.register(models.Form496Part1Item)
class Form496Part1ItemAdmin(BaseAdmin):
    """
    Custom admin for the Form496Part1Item model.
    """
    list_display = (
        'filing',
        "candidate_title",
        "candidate_lastname",
        "candidate_firstname",
        "candidate_name_suffix",
        "candidate_office_code",
        "ballot_measure_name",
        "ballot_measure_number",
        "ballot_measure_jurisdiction",
        "support_opposition_code"
    )


@admin.register(models.Form496Part1ItemVersion)
class Form496Part1ItemVersionAdmin(BaseAdmin):
    """
    Custom admin for the Form496Part1ItemVersion model.
    """
    list_display = (
        'filing_version',
        "candidate_title",
        "candidate_lastname",
        "candidate_firstname",
        "candidate_name_suffix",
        "candidate_office_code",
        "ballot_measure_name",
        "ballot_measure_number",
        "ballot_measure_jurisdiction",
        "support_opposition_code"
    )


@admin.register(models.Form496Part2Item)
class Form496Part2ItemAdmin(BaseAdmin):
    """
    Custom admin for the Form496Part2Item model.
    """
    list_display = (
        'filing',
        "line_item",
        "expense_date",
        "amount",
        "transaction_id",
    )


@admin.register(models.Form496Part2ItemVersion)
class Form496Part2ItemVersionAdmin(BaseAdmin):
    """
    Custom admin for the Form496Part2ItemVersion model.
    """
    list_display = (
        'filing_version',
        "line_item",
        "expense_date",
        "amount",
        "transaction_id",
    )


@admin.register(models.Form496Part3Item)
class Form496Part3ItemAdmin(BaseAdmin):
    """
    Custom admin for the Form496Part3Item model.
    """
    list_display = (
        'filing',
        'line_item',
        'date_received',
        'amount',
        'transaction_id',
        'contributor_code',
        'contributor_lastname',
        'contributor_firstname',
    )


@admin.register(models.Form496Part3ItemVersion)
class Form496Part3ItemVersionAdmin(BaseAdmin):
    """
    Custom admin for the Form496Part3ItemVersion model.
    """
    list_display = (
        'filing_version',
        'line_item',
        'date_received',
        'amount',
        'transaction_id',
        'contributor_code',
        'contributor_lastname',
        'contributor_firstname',
    )



# -*- coding: utf-8 -*-
# Generated by Django 1.11.8 on 2018-04-16 22:00
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('calaccess_processed', '__first__'),
        ('calaccess_processed', '0014_pgcrypto'),
    ]

    operations = [
        migrations.DeleteModel(
            name='FilerIDValue',
        ),
        migrations.DeleteModel(
            name='FilingIDValue',
        ),
        migrations.AlterUniqueTogether(
            name='form460filingversion',
            unique_together=set([]),
        ),
        migrations.AlterIndexTogether(
            name='form460filingversion',
            index_together=set([]),
        ),
        migrations.RemoveField(
            model_name='form460filingversion',
            name='filing',
        ),
        migrations.AlterUniqueTogether(
            name='form460scheduleaitem',
            unique_together=set([]),
        ),
        migrations.RemoveField(
            model_name='form460scheduleaitem',
            name='filing',
        ),
        migrations.AlterUniqueTogether(
            name='form460scheduleaitemversion',
            unique_together=set([]),
        ),
        migrations.AlterIndexTogether(
            name='form460scheduleaitemversion',
            index_together=set([]),
        ),
        migrations.RemoveField(
            model_name='form460scheduleaitemversion',
            name='filing_version',
        ),
        migrations.RemoveField(
            model_name='form460scheduleasummary',
            name='filing',
        ),
        migrations.RemoveField(
            model_name='form460scheduleasummaryversion',
            name='filing_version',
        ),
        migrations.AlterUniqueTogether(
            name='form460scheduleb1item',
            unique_together=set([]),
        ),
        migrations.RemoveField(
            model_name='form460scheduleb1item',
            name='filing',
        ),
        migrations.AlterUniqueTogether(
            name='form460scheduleb1itemversion',
            unique_together=set([]),
        ),
        migrations.AlterIndexTogether(
            name='form460scheduleb1itemversion',
            index_together=set([]),
        ),
        migrations.RemoveField(
            model_name='form460scheduleb1itemversion',
            name='filing_version',
        ),
        migrations.AlterUniqueTogether(
            name='form460scheduleb2item',
            unique_together=set([]),
        ),
        migrations.RemoveField(
            model_name='form460scheduleb2item',
            name='filing',
        ),
        migrations.AlterUniqueTogether(
            name='form460scheduleb2itemold',
            unique_together=set([]),
        ),
        migrations.RemoveField(
            model_name='form460scheduleb2itemold',
            name='filing',
        ),
        migrations.AlterUniqueTogether(
            name='form460scheduleb2itemversion',
            unique_together=set([]),
        ),
        migrations.AlterIndexTogether(
            name='form460scheduleb2itemversion',
            index_together=set([]),
        ),
        migrations.RemoveField(
            model_name='form460scheduleb2itemversion',
            name='filing_version',
        ),
        migrations.AlterUniqueTogether(
            name='form460scheduleb2itemversionold',
            unique_together=set([]),
        ),
        migrations.AlterIndexTogether(
            name='form460scheduleb2itemversionold',
            index_together=set([]),
        ),
        migrations.RemoveField(
            model_name='form460scheduleb2itemversionold',
            name='filing_version',
        ),
        migrations.AlterUniqueTogether(
            name='form460schedulecitem',
            unique_together=set([]),
        ),
        migrations.RemoveField(
            model_name='form460schedulecitem',
            name='filing',
        ),
        migrations.AlterUniqueTogether(
            name='form460schedulecitemversion',
            unique_together=set([]),
        ),
        migrations.AlterIndexTogether(
            name='form460schedulecitemversion',
            index_together=set([]),
        ),
        migrations.RemoveField(
            model_name='form460schedulecitemversion',
            name='filing_version',
        ),
        migrations.RemoveField(
            model_name='form460schedulecsummary',
            name='filing',
        ),
        migrations.RemoveField(
            model_name='form460schedulecsummaryversion',
            name='filing_version',
        ),
        migrations.AlterUniqueTogether(
            name='form460scheduleditem',
            unique_together=set([]),
        ),
        migrations.RemoveField(
            model_name='form460scheduleditem',
            name='filing',
        ),
        migrations.AlterUniqueTogether(
            name='form460scheduleditemversion',
            unique_together=set([]),
        ),
        migrations.AlterIndexTogether(
            name='form460scheduleditemversion',
            index_together=set([]),
        ),
        migrations.RemoveField(
            model_name='form460scheduleditemversion',
            name='filing_version',
        ),
        migrations.AlterUniqueTogether(
            name='form460scheduleeitem',
            unique_together=set([]),
        ),
        migrations.RemoveField(
            model_name='form460scheduleeitem',
            name='filing',
        ),
        migrations.AlterUniqueTogether(
            name='form460scheduleeitemversion',
            unique_together=set([]),
        ),
        migrations.AlterIndexTogether(
            name='form460scheduleeitemversion',
            index_together=set([]),
        ),
        migrations.RemoveField(
            model_name='form460scheduleeitemversion',
            name='filing_version',
        ),
        migrations.AlterUniqueTogether(
            name='form460scheduleesubitem',
            unique_together=set([]),
        ),
        migrations.RemoveField(
            model_name='form460scheduleesubitem',
            name='filing',
        ),
        migrations.AlterUniqueTogether(
            name='form460scheduleesubitemversion',
            unique_together=set([]),
        ),
        migrations.AlterIndexTogether(
            name='form460scheduleesubitemversion',
            index_together=set([]),
        ),
        migrations.RemoveField(
            model_name='form460scheduleesubitemversion',
            name='filing_version',
        ),
        migrations.RemoveField(
            model_name='form460scheduleesummary',
            name='filing',
        ),
        migrations.RemoveField(
            model_name='form460scheduleesummaryversion',
            name='filing_version',
        ),
        migrations.AlterUniqueTogether(
            name='form460schedulefitem',
            unique_together=set([]),
        ),
        migrations.RemoveField(
            model_name='form460schedulefitem',
            name='filing',
        ),
        migrations.AlterUniqueTogether(
            name='form460schedulefitemversion',
            unique_together=set([]),
        ),
        migrations.AlterIndexTogether(
            name='form460schedulefitemversion',
            index_together=set([]),
        ),
        migrations.RemoveField(
            model_name='form460schedulefitemversion',
            name='filing_version',
        ),
        migrations.AlterUniqueTogether(
            name='form460schedulegitem',
            unique_together=set([]),
        ),
        migrations.RemoveField(
            model_name='form460schedulegitem',
            name='filing',
        ),
        migrations.AlterUniqueTogether(
            name='form460schedulegitemversion',
            unique_together=set([]),
        ),
        migrations.AlterIndexTogether(
            name='form460schedulegitemversion',
            index_together=set([]),
        ),
        migrations.RemoveField(
            model_name='form460schedulegitemversion',
            name='filing_version',
        ),
        migrations.AlterUniqueTogether(
            name='form460scheduleh2itemold',
            unique_together=set([]),
        ),
        migrations.RemoveField(
            model_name='form460scheduleh2itemold',
            name='filing',
        ),
        migrations.AlterUniqueTogether(
            name='form460scheduleh2itemversionold',
            unique_together=set([]),
        ),
        migrations.AlterIndexTogether(
            name='form460scheduleh2itemversionold',
            index_together=set([]),
        ),
        migrations.RemoveField(
            model_name='form460scheduleh2itemversionold',
            name='filing_version',
        ),
        migrations.AlterUniqueTogether(
            name='form460schedulehitem',
            unique_together=set([]),
        ),
        migrations.RemoveField(
            model_name='form460schedulehitem',
            name='filing',
        ),
        migrations.AlterUniqueTogether(
            name='form460schedulehitemversion',
            unique_together=set([]),
        ),
        migrations.AlterIndexTogether(
            name='form460schedulehitemversion',
            index_together=set([]),
        ),
        migrations.RemoveField(
            model_name='form460schedulehitemversion',
            name='filing_version',
        ),
        migrations.AlterUniqueTogether(
            name='form460scheduleiitem',
            unique_together=set([]),
        ),
        migrations.RemoveField(
            model_name='form460scheduleiitem',
            name='filing',
        ),
        migrations.AlterUniqueTogether(
            name='form460scheduleiitemversion',
            unique_together=set([]),
        ),
        migrations.AlterIndexTogether(
            name='form460scheduleiitemversion',
            index_together=set([]),
        ),
        migrations.RemoveField(
            model_name='form460scheduleiitemversion',
            name='filing_version',
        ),
        migrations.AlterUniqueTogether(
            name='form497filingversion',
            unique_together=set([]),
        ),
        migrations.AlterIndexTogether(
            name='form497filingversion',
            index_together=set([]),
        ),
        migrations.RemoveField(
            model_name='form497filingversion',
            name='filing',
        ),
        migrations.AlterUniqueTogether(
            name='form497part1item',
            unique_together=set([]),
        ),
        migrations.RemoveField(
            model_name='form497part1item',
            name='filing',
        ),
        migrations.AlterUniqueTogether(
            name='form497part1itemversion',
            unique_together=set([]),
        ),
        migrations.AlterIndexTogether(
            name='form497part1itemversion',
            index_together=set([]),
        ),
        migrations.RemoveField(
            model_name='form497part1itemversion',
            name='filing_version',
        ),
        migrations.AlterUniqueTogether(
            name='form497part2item',
            unique_together=set([]),
        ),
        migrations.RemoveField(
            model_name='form497part2item',
            name='filing',
        ),
        migrations.AlterUniqueTogether(
            name='form497part2itemversion',
            unique_together=set([]),
        ),
        migrations.AlterIndexTogether(
            name='form497part2itemversion',
            index_together=set([]),
        ),
        migrations.RemoveField(
            model_name='form497part2itemversion',
            name='filing_version',
        ),
        migrations.AlterUniqueTogether(
            name='form501filingversion',
            unique_together=set([]),
        ),
        migrations.AlterIndexTogether(
            name='form501filingversion',
            index_together=set([]),
        ),
        migrations.RemoveField(
            model_name='form501filingversion',
            name='filing',
        ),
        migrations.DeleteModel(
            name='OCDBallotMeasureContestIdentifierProxy',
        ),
        migrations.DeleteModel(
            name='OCDBallotMeasureContestOptionProxy',
        ),
        migrations.DeleteModel(
            name='OCDBallotMeasureContestProxy',
        ),
        migrations.DeleteModel(
            name='OCDBallotMeasureContestSourceProxy',
        ),
        migrations.DeleteModel(
            name='OCDCandidacyProxy',
        ),
        migrations.DeleteModel(
            name='OCDCandidacySourceProxy',
        ),
        migrations.DeleteModel(
            name='OCDCandidateContestPostProxy',
        ),
        migrations.DeleteModel(
            name='OCDCandidateContestProxy',
        ),
        migrations.DeleteModel(
            name='OCDCandidateContestSourceProxy',
        ),
        migrations.DeleteModel(
            name='OCDDivisionProxy',
        ),
        migrations.DeleteModel(
            name='OCDElectionIdentifierProxy',
        ),
        migrations.DeleteModel(
            name='OCDElectionProxy',
        ),
        migrations.DeleteModel(
            name='OCDElectionSourceProxy',
        ),
        migrations.DeleteModel(
            name='OCDFlatBallotMeasureContestProxy',
        ),
        migrations.DeleteModel(
            name='OCDFlatCandidacyProxy',
        ),
        migrations.DeleteModel(
            name='OCDFlatRetentionContestProxy',
        ),
        migrations.DeleteModel(
            name='OCDMembershipProxy',
        ),
        migrations.DeleteModel(
            name='OCDOrganizationIdentifierProxy',
        ),
        migrations.DeleteModel(
            name='OCDOrganizationNameProxy',
        ),
        migrations.DeleteModel(
            name='OCDOrganizationProxy',
        ),
        migrations.DeleteModel(
            name='OCDPartyProxy',
        ),
        migrations.DeleteModel(
            name='OCDPersonIdentifierProxy',
        ),
        migrations.DeleteModel(
            name='OCDPersonNameProxy',
        ),
        migrations.DeleteModel(
            name='OCDPersonProxy',
        ),
        migrations.DeleteModel(
            name='OCDPostProxy',
        ),
        migrations.DeleteModel(
            name='OCDRetentionContestIdentifierProxy',
        ),
        migrations.DeleteModel(
            name='OCDRetentionContestOptionProxy',
        ),
        migrations.DeleteModel(
            name='OCDRetentionContestProxy',
        ),
        migrations.DeleteModel(
            name='OCDRetentionContestSourceProxy',
        ),
        migrations.DeleteModel(
            name='RawFilerToFilerTypeCdProxy',
        ),
        migrations.DeleteModel(
            name='ScrapedCandidateElectionProxy',
        ),
        migrations.DeleteModel(
            name='ScrapedCandidateProxy',
        ),
        migrations.DeleteModel(
            name='ScrapedIncumbentElectionProxy',
        ),
        migrations.DeleteModel(
            name='ScrapedIncumbentProxy',
        ),
        migrations.DeleteModel(
            name='ScrapedPropositionElectionProxy',
        ),
        migrations.DeleteModel(
            name='ScrapedPropositionProxy',
        ),
        migrations.DeleteModel(
            name='Form460FilingVersion',
        ),
        migrations.DeleteModel(
            name='Form460ScheduleAItem',
        ),
        migrations.DeleteModel(
            name='Form460ScheduleAItemVersion',
        ),
        migrations.DeleteModel(
            name='Form460ScheduleASummary',
        ),
        migrations.DeleteModel(
            name='Form460ScheduleASummaryVersion',
        ),
        migrations.DeleteModel(
            name='Form460ScheduleB1Item',
        ),
        migrations.DeleteModel(
            name='Form460ScheduleB1ItemVersion',
        ),
        migrations.DeleteModel(
            name='Form460ScheduleB2Item',
        ),
        migrations.DeleteModel(
            name='Form460ScheduleB2ItemOld',
        ),
        migrations.DeleteModel(
            name='Form460ScheduleB2ItemVersion',
        ),
        migrations.DeleteModel(
            name='Form460ScheduleB2ItemVersionOld',
        ),
        migrations.DeleteModel(
            name='Form460ScheduleCItem',
        ),
        migrations.DeleteModel(
            name='Form460ScheduleCItemVersion',
        ),
        migrations.DeleteModel(
            name='Form460ScheduleCSummary',
        ),
        migrations.DeleteModel(
            name='Form460ScheduleCSummaryVersion',
        ),
        migrations.DeleteModel(
            name='Form460ScheduleDItem',
        ),
        migrations.DeleteModel(
            name='Form460ScheduleDItemVersion',
        ),
        migrations.DeleteModel(
            name='Form460ScheduleEItem',
        ),
        migrations.DeleteModel(
            name='Form460ScheduleEItemVersion',
        ),
        migrations.DeleteModel(
            name='Form460ScheduleESubItem',
        ),
        migrations.DeleteModel(
            name='Form460ScheduleESubItemVersion',
        ),
        migrations.DeleteModel(
            name='Form460ScheduleESummary',
        ),
        migrations.DeleteModel(
            name='Form460ScheduleESummaryVersion',
        ),
        migrations.DeleteModel(
            name='Form460ScheduleFItem',
        ),
        migrations.DeleteModel(
            name='Form460ScheduleFItemVersion',
        ),
        migrations.DeleteModel(
            name='Form460ScheduleGItem',
        ),
        migrations.DeleteModel(
            name='Form460ScheduleGItemVersion',
        ),
        migrations.DeleteModel(
            name='Form460ScheduleH2ItemOld',
        ),
        migrations.DeleteModel(
            name='Form460ScheduleH2ItemVersionOld',
        ),
        migrations.DeleteModel(
            name='Form460ScheduleHItem',
        ),
        migrations.DeleteModel(
            name='Form460ScheduleHItemVersion',
        ),
        migrations.DeleteModel(
            name='Form460ScheduleIItem',
        ),
        migrations.DeleteModel(
            name='Form460ScheduleIItemVersion',
        ),
        migrations.DeleteModel(
            name='Form497FilingVersion',
        ),
        migrations.DeleteModel(
            name='Form497Part1Item',
        ),
        migrations.DeleteModel(
            name='Form497Part1ItemVersion',
        ),
        migrations.DeleteModel(
            name='Form497Part2Item',
        ),
        migrations.DeleteModel(
            name='Form497Part2ItemVersion',
        ),
        migrations.DeleteModel(
            name='Form501FilingVersion',
        ),
        migrations.DeleteModel(
            name='Form501Filing',
        ),
        migrations.DeleteModel(
            name='Form497Filing',
        ),
        migrations.DeleteModel(
            name='Form460Filing',
        ),
    ]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Models for storing data from Campaign Disclosure Statements (Form 460).
"""
from django.db import models
from calaccess_processed_filings.models.campaign import CampaignExpenditureItemBase


class Form460ScheduleDItemBase(CampaignExpenditureItemBase):
    """
    Abstract base model for items reported on Schedule D of Form 460 filings.

    On Schedule D, campaign filers are required to summarize contributions
    and independent expenditures in support or opposition to other candidates
    and ballot measures.
    """
    cumulative_election_amount = models.DecimalField(
        decimal_places=2,
        max_digits=14,
        null=True,
        help_text="If the candidate is subject to contribution limits, the "
                  "cumulative amount given by the filer during the election "
                  "cycle as of the Form 460's filing date (from EXPN_CD."
                  "CUM_OTH)"
    )

    class Meta:
        """
        Model options.
        """
        app_label = 'calaccess_processed_filings'
        abstract = True


class Form460ScheduleDItem(Form460ScheduleDItemBase):
    """
    Payments in support or opposition of other candidates and ballot measures.

    These transactions are itemized on Schedule D of the most recent version
    of each Form 460 filing. For payments itemized on any version of any Form
    460 filing, see Form460ScheduleDItemVersion.

    Derived from EXPN_CD records where FORM_TYPE is 'D'.
    """
    filing = models.ForeignKey(
        'Form460Filing',
        related_name='schedule_d_items',
        null=True,
        on_delete=models.SET_NULL,
        help_text='Foreign key referring to the Form 460 on which the '
                  'payment was reported (from EXPN_CD.FILING_ID)',
    )

    class Meta:
        """
        Model options.
        """
        app_label = 'calaccess_processed_filings'
        unique_together = ((
            'filing',
            'line_item',
        ),)
        verbose_name = "Form 460 (Campaign Disclosure) Schedule D item"

    def __str__(self):
        return '%s-%s' % (self.filing, self.line_item)


class Form460ScheduleDItemVersion(Form460ScheduleDItemBase):
    """
    Every version of each payment supporting/opposing another candidate/ballot measure.

    For payments itemized on Schedule D of the most recent version of each Form
    460 filing, see Form460ScheduleDItem.

    Derived from EXPN_CD records where FORM_TYPE is 'D'.
    """
    filing_version = models.ForeignKey(
        'Form460FilingVersion',
        related_name='schedule_d_items',
        null=True,
        on_delete=models.SET_NULL,
        help_text='Foreign key referring to the version of the Form 460 that '
                  'includes the payment made'
    )

    class Meta:
        """
        Model options.
        """
        app_label = 'calaccess_processed_filings'
        unique_together = ((
            'filing_version',
            'line_item',
        ),)
        index_together = ((
            'filing_version',
            'line_item',
        ),)
        verbose_name = "Form 460 (Campaign Disclosure) Schedule D item version"

    def __str__(self):
        return '%s-%s-%s' % (
            self.filing_version.filing_id,
            self.filing_version.amend_id,
            self.line_item
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Load data into processed CAL-ACCESS models, archive processed files and ZIP.
"""
# Files
import os
from zipfile import ZIP_DEFLATED, ZIP_STORED, ZipFile

# Commands
from django.core.management import call_command
from django.core.management.base import CommandError
from calaccess_processed.management.commands import CalAccessCommand

# Models
from calaccess_scraped.models import PropositionElection

# Misc
from django.conf import settings
from django.utils.timezone import now


class Command(CalAccessCommand):
    """
    Load data into processed CAL-ACCESS models, archive processed files and ZIP.
    """
    help = 'Load data into processed CAL-ACCESS models, archive processed files and ZIP.'

    def add_arguments(self, parser):
        """
        Adds custom arguments specific to this command.
        """
        parser.add_argument(
            "--force-restart",
            "--restart",
            action="store_true",
            dest="restart",
            default=False,
            help="Force re-start (overrides auto-resume)."
        )

    def handle(self, *args, **options):
        """
        Make it happen.
        """
        # Throw an error if the scraper hasn't been run.
        if not PropositionElection.objects.exists():
            raise CommandError("Sorry. You must first run 'scrapecalaccess' from the calaccess_scraped app.")

        # Set options
        super(Command, self).handle(*args, **options)
        self.force_restart = options.get("restart")

        # Get or create the logger record
        self.processed_version, created = self.get_or_create_processed_version()

        # If the version is already fully processed and we're not forcing a do-over
        # then just quit out now
        if self.processed_version.update_completed and not self.force_restart:
            msg_tmp = 'Processing completed at %s.'
            self.success(
                msg_tmp % self.processed_version.process_finish_datetime.ctime()
            )
            return False

        # Otherwise proceed with the standard routine
        # First, start the clock if created (or restart if forcing restart)
        if created or self.force_restart:
            self.processed_version.process_start_datetime = now()
            # also reset finish time if forcing re-start
            if self.force_restart:
                self.processed_version.process_finish_datetime = None
            self.processed_version.save()
            if self.verbosity > 2:
                self.log('Flushing local copies of processed data files.')
            self.flush_data_files()

        # then load
        self.load()

        # then verify
        call_command(
            'verifycalaccessprocesseddata',
            verbosity=self.verbosity,
            no_color=self.no_color,
        )

        if getattr(settings, 'CALACCESS_STORE_ARCHIVE', False):
            # then zip
            self.zip('flat')
            self.zip('relational')

        # Wrap up the log
        self.processed_version.process_finish_datetime = now()
        self.processed_version.save()
        self.success('Processing complete')
        self.duration()

    def flush_data_files(self):
        """
        Delete files in processed_data_dir, prior to archiving.
        """
        for dirpath, dirnames, filenames in os.walk(self.processed_data_dir):
            file_paths = [os.path.join(dirpath, i) for i in filenames]
            for file_path in file_paths:
                try:
                    os.remove(file_path)
                except OSError:
                    pass

    def load(self):
        """
        Load all of the processed models.
        """
        call_command(
            'processcalaccessfilings',
            verbosity=self.verbosity,
            no_color=self.no_color,
            force_restart=self.force_restart
        )
        call_command(
            'processcalaccesselections',
            verbosity=self.verbosity,
            no_color=self.no_color,
        )
        call_command(
            'processcalaccessflatfiles',
            verbosity=self.verbosity,
            no_color=self.no_color,
        )

    def zip(self, directory_name):
        """
        Zip up files in directory_name (in processed_data_dir).
        """
        directory = os.path.join(self.processed_data_dir, directory_name)
        if self.verbosity:
            self.log("Zipping files in %s/" % directory)

        # enable zipfile compression
        compression = ZIP_DEFLATED
        zip_path = os.path.join(self.data_dir, '%s.zip' % directory_name)

        try:
            zf = ZipFile(zip_path, 'w', compression, allowZip64=True)
        except RuntimeError:
            self.error('Zip file cannot be compressed (check zlib module).')
            compression = ZIP_STORED
            zf = ZipFile(zip_path, 'w', compression, allowZip64=True)

        # loop over and save files in csv processed data dir
        for f in os.listdir(directory):
            if self.verbosity > 2:
                self.log(" Adding %s to zip" % f)
            csv_path = os.path.join(directory, f)
            zf.write(csv_path, f)

        # close the zip file
        zf.close()

        return zip_path



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Proxy models for OCD's campaign finance models.
"""
from .committees import (
    OCDCommitteeProxy,
    OCDCommitteeIdentifierProxy,
    OCDCommitteeNameProxy,
    OCDCommitteeSourceProxy,
    OCDCommitteeTypeProxy
)
from .filings import (
    OCDFilingProxy,
    OCDFilingIdentifierProxy,
    OCDFilingSourceProxy,
    OCDFilingActionProxy,
    OCDFilingActionSummaryAmountProxy
)
from .transactions import (
    OCDTransactionProxy,
    OCDTransactionIdentifierProxy
)

__all__ = (
    'OCDCommitteeProxy',
    'OCDCommitteeIdentifierProxy',
    'OCDCommitteeNameProxy',
    'OCDCommitteeSourceProxy',
    'OCDCommitteeTypeProxy',
    'OCDFilingProxy',
    'OCDFilingIdentifierProxy',
    'OCDFilingSourceProxy',
    'OCDFilingActionProxy',
    'OCDFilingActionSummaryAmountProxy',
    'OCDTransactionProxy',
    'OCDTransactionIdentifierProxy',
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Custom administration panels for campaign entity models.
"""
from __future__ import unicode_literals
from django.contrib import admin
from calaccess_processed_filings import models
from calaccess_raw.admin.base import BaseAdmin


@admin.register(models.Form501Filing)
class Form501FilingAdmin(BaseAdmin):
    """
    Custom admin for the Form501Filing model.
    """
    list_display = (
        'filing_id',
        'amendment_count',
        'date_filed',
        'committee_id',
        'last_name',
        'first_name',
        'office',
        'district',
        'party',
        'jurisdiction',
        'election_type',
        'election_year',
    )


@admin.register(models.Form501FilingVersion)
class Form501FilingVersionAdmin(BaseAdmin):
    """
    Custom admin for the Form501FilingVersion model.
    """
    list_display = (
        'filing',
        'amend_id',
        'date_filed',
        'committee_id',
        'last_name',
        'first_name',
        'office',
        'district',
        'party',
        'jurisdiction',
        'election_type',
        'election_year',
    )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Load and archive the CAL-ACCESS Filing and FilingVersion models.
"""
import os
from django.apps import apps
from django.conf import settings
from django.db import connection
from django.utils.timezone import now
from django.core.management import call_command
from calaccess_processed.management.commands import CalAccessCommand
from calaccess_processed.models.tracking import ProcessedDataFile


class Command(CalAccessCommand):
    """
    Load and archive the CAL-ACCESS Filing and FilingVersion models.
    """
    help = 'Load and archive the CAL-ACCESS Filing and FilingVersion models.'

    def add_arguments(self, parser):
        """
        Adds custom arguments specific to this command.
        """
        parser.add_argument(
            "--force-restart",
            "--restart",
            action="store_true",
            dest="restart",
            default=False,
            help="Force re-start (overrides auto-resume)."
        )

    def handle(self, *args, **options):
        """
        Make it happen.
        """
        super(Command, self).handle(*args, **options)

        self.force_restart = options.get("restart")

        # get or create the ProcessedDataVersion instance
        self.processed_version, created = self.get_or_create_processed_version()

        if self.processed_version.files.count() > 0 and not self.force_restart:
            self.header(
                'Resume loading of filings from {:%m-%d-%Y %H:%M:%S} snapshot'.format(
                    self.processed_version.raw_version.release_datetime,
                )
            )
        else:
            self.header(
                'Load filings from {:%m-%d-%Y %H:%M:%S} snapshot'.format(
                    self.processed_version.raw_version.release_datetime
                )
            )
        # set the time if not there already or forcing restart
        if not self.processed_version.process_start_datetime or self.force_restart:
            self.processed_version.process_start_datetime = now()
            self.processed_version.save()

        # create subdirectory in processed_data_dir, if missing
        filings_data_path = os.path.join(self.processed_data_dir, 'filings')
        os.path.isdir(filings_data_path) or os.makedirs(filings_data_path)

        self.handle_models('version')
        self.handle_models('filing')

    def handle_models(self, model_type):
        """
        Handle logic for loading models of model_type.
        """
        model_list = self.get_model_list(model_type)
        if len(model_list) > 0:
            if self.verbosity >= 2:
                self.log(" Loading {} {} models.".format(len(model_list), model_type))
            self.load_model_list(model_list)

            # archive if django project setting enabled
            if getattr(settings, 'CALACCESS_STORE_ARCHIVE', False):
                self.archive_model_list(model_list)

    def get_model_list(self, model_type):
        """
        Return a list of models of the specified type to be loaded.

        model_type must be "version" of "filing".
        """
        model_list = apps.get_app_config('calaccess_processed_filings').get_filing_models()

        if model_type == 'version':
            models_to_load = [m for m in model_list if 'Version' in str(m)]
        elif model_type == 'filing':
            models_to_load = [m for m in model_list if 'Version' not in str(m)]
        else:
            raise ValueError('model_type must be "version" or "filing".')

        # if not forcing a restart, filter out the models already loaded
        if not self.force_restart:
            loaded_models_q = ProcessedDataFile.objects.filter(
                version=self.processed_version,
                process_finish_datetime__isnull=False,
            )
            if model_type == 'version':
                loaded_models_q.filter(file_name__icontains='Version')
            elif model_type == 'filing':
                loaded_models_q.filter(
                    file_name__icontains='Form'
                ).exclude(file_name__icontains='Version')

            loaded_models = [i.file_name for i in loaded_models_q.all()]

            if self.verbosity >= 2:
                self.log(
                    " {0} {1} models already loaded.".format(
                        len(loaded_models),
                        model_type,
                    )
                )
            models_to_load = [
                m for m in models_to_load
                if m._meta.object_name not in loaded_models
            ]

        return models_to_load

    def load_model_list(self, model_list):
        """
        Iterate over the given list of models, loading each one.
        """
        # iterate over all of filing models
        for m in model_list:

            # set up the ProcessedDataFile instance
            processed_file, created = ProcessedDataFile.objects.get_or_create(
                version=self.processed_version,
                file_name=m._meta.object_name,
            )
            processed_file.process_start_datetime = now()
            processed_file.save()

            # flush the processed model
            if self.verbosity > 2:
                self.log(" Truncating %s" % m._meta.db_table)
            with connection.cursor() as c:
                c.execute('TRUNCATE TABLE "%s" RESTART IDENTITY CASCADE' % (m._meta.db_table))

            # load the processed model
            if self.verbosity > 2:
                self.log(" Loading %s" % m._meta.db_table)
            m.objects.load()

            processed_file.records_count = m.objects.count()
            processed_file.process_finish_datetime = now()
            processed_file.save()

    def archive_model_list(self, model_list):
        """
        Iterate over the given list of models, archiving each one.
        """
        for m in model_list:
            call_command('archivecalaccessfilingsfile', m._meta.object_name)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Models for storing data from Campaign Disclosure Statements (Form 460).
"""
from django.db import models
from calaccess_processed_filings.models.campaign import CampaignLoanMadeItemBase


class Form460ScheduleHItemBase(CampaignLoanMadeItemBase):
    """
    Abstract base model for items reported on Schedule H of Form 460.

    On Schedule H, campaign filers are required to report loans made or
    currently outstanding to other recipients during the period covered by the
    filing.
    """
    begin_period_balance = models.DecimalField(
        verbose_name='beginning period balance',
        decimal_places=2,
        max_digits=14,
        help_text="Outstanding balance of the loan at the beginning of the"
                  "period covered by the filing (from LOAN_CD.LOAN_AMT4)"
    )
    amount_loaned = models.DecimalField(
        verbose_name='amount loaned',
        decimal_places=2,
        max_digits=14,
        help_text="Amount loaned during the period covered by the filing "
                  "(from LOAN_CD.LOAN_AMT1)"
    )
    amount_paid = models.DecimalField(
        verbose_name='amount paid',
        decimal_places=2,
        max_digits=14,
        help_text="Amount paid back during the period covered by the filing "
                  "(from LOAN_CD.LOAN_AMT5)"
    )
    amount_forgiven = models.DecimalField(
        verbose_name='amount forgiven',
        decimal_places=2,
        max_digits=14,
        help_text="Amount forgiven by the campaign filer during the period "
                  "covered by the filing (from LOAN_CD.LOAN_AMT6)"
    )
    end_period_balance = models.DecimalField(
        verbose_name='end period balance',
        decimal_places=2,
        max_digits=14,
        help_text="Outstanding balance of the loan at the end of the period "
                  "covered by the filing (from LOAN_CD.LOAN_AMT2)"
    )
    date_due = models.DateField(
        verbose_name='date due',
        null=True,
        help_text="Date that the loan is due (from LOAN_CD.LOAN_DATE2)"
    )
    interest_received = models.DecimalField(
        verbose_name='interest paid',
        decimal_places=2,
        max_digits=14,
        help_text="Amount of interest paid on the loan during the period "
                  "covered by the campaign filing (from LOAN_CD.LOAN_AMT7)"
    )
    original_amount = models.DecimalField(
        verbose_name='original amount',
        decimal_places=2,
        max_digits=14,
        help_text="Original amount loaned by the lender to the campaign filer "
                  "(from LOAN_CD.LOAN_AMT8)"
    )
    date_incurred = models.DateField(
        verbose_name='date incurred',
        null=True,
        help_text="Date the loan was made or received (from LOAN_CD.LOAN_DATE1)"
    )
    cumulative_ytd_contributions = models.DecimalField(
        verbose_name='cumulative year-to-date contributions',
        decimal_places=2,
        max_digits=14,
        help_text="Cumulative amount of contributions (loans, monetary and "
                  "nonmonetary contributions) from the campaign filer to the "
                  "recipient during the calendar year covered by this statement"
                  " (from LOAN_CD.LOAN_AMT3)"
    )
    reported_on_h1 = models.BooleanField(
        verbose_name='reported on H1',
        default=False,
        help_text='Indicates if the item was actually reported on Part 1 of '
                  'Schedule H. Until 2001, campaign filers were required to '
                  'report loans made to others on Part 1 of Schedule H, '
                  'separate from repayments or forgiveness of those loans '
                  '(Schedule H, Part 2)'
    )

    class Meta:
        """
        Model options.
        """
        app_label = 'calaccess_processed_filings'
        abstract = True


class Form460ScheduleHItem(Form460ScheduleHItemBase):
    """
    Loans made by campaign filers to other recipients.

    These transactions are itemized on Schedule H of the most recent version of
    each Form 460 filing. For loans itemized on any version of any Form 460
    filing, see Form460ScheduleHItemVersion.

    Derived from LOAN_CD records where FORM_TYPE is 'H' or 'H1'.
    """
    filing = models.ForeignKey(
        'Form460Filing',
        related_name='schedule_h_items',
        null=True,
        on_delete=models.SET_NULL,
        help_text='Foreign key referring to the Form 460 on which the loan '
                  'was reported (from LOAN_CD.FILING_ID)',
    )

    class Meta:
        """
        Model options.
        """
        app_label = 'calaccess_processed_filings'
        unique_together = ((
            'filing',
            'line_item',
        ),)
        verbose_name = "Form 460 (Campaign Disclosure) Schedule H item"

    def __str__(self):
        return '%s-%s' % (self.filing, self.line_item)


class Form460ScheduleHItemVersion(Form460ScheduleHItemBase):
    """
    Every version of each loan made by a campaign filer another recipient.

    For outstanding loans itemized on Schedule H of the most recent version of
    each Form 460 filing, see Form460ScheduleHItem.

    Derived from LOAN_CD records where FORM_TYPE is 'H' or 'H1'.
    """
    filing_version = models.ForeignKey(
        'Form460FilingVersion',
        related_name='schedule_h_items',
        null=True,
        on_delete=models.SET_NULL,
        help_text='Foreign key referring to the version of the Form 460 that '
                  'includes the outstanding loan'
    )

    class Meta:
        """
        Model options.
        """
        app_label = 'calaccess_processed_filings'
        unique_together = ((
            'filing_version',
            'line_item',
        ),)
        index_together = ((
            'filing_version',
            'line_item',
        ),)
        verbose_name = "Form 460 (Campaign Disclosure) Schedule H item version"

    def __str__(self):
        return '%s-%s-%s' % (
            self.filing_version.filing_id,
            self.filing_version.amend_id,
            self.line_item
        )


class Form460ScheduleH2ItemBaseOld(CampaignLoanMadeItemBase):
    """
    Abstract base model for Schedule H, Part 2, items from Form 460 circa 2001.

    Until Form 460 was modified in 2001, campaign filers were required to report
    repayments on and forgiveness of loans made to others on Part 2 of Schedule
    H.
    """
    date_repaid_or_forgiven = models.DateField(
        verbose_name='date paid or forgiven',
        null=True,
        help_text="Date when the loan repayment or forgiveness occurred (from "
                  "LOAN_CD.LOAN_DATE2)"
    )
    date_of_original_loan = models.DateField(
        verbose_name='date of original loan',
        null=True,
        help_text="Date the loan was orginally made (from LOAN_CD.LOAN_DATE1)"
    )
    REPAYMENT_TYPE_CHOICES = (
        ('H2F', 'Forgiven'),
        ('H2R', 'Repay'),
        ('H2T', 'Third party payment'),
    )
    repayment_type = models.CharField(
        verbose_name='repayment type',
        max_length=3,
        choices=REPAYMENT_TYPE_CHOICES,
        help_text='Indicates whether the item is a loan repayment to the '
                  'campaign filer, a repayment by a third-party or a loan '
                  'forgiveness by the campaign filer (from LOAN_CD.LOAN_TYPE)',
    )
    amount_repaid_or_forgiven = models.DecimalField(
        verbose_name='amount repaid or forgiven',
        decimal_places=2,
        max_digits=14,
        help_text="Amount paid back or forgiven during the period covered by "
                  "the filing (from LOAN_CD.LOAN_AMT1)"
    )
    outstanding_principle = models.DecimalField(
        verbose_name='outstanding principle',
        decimal_places=2,
        max_digits=14,
        help_text="Outstanding balance of the loan at the end of the period "
                  "covered by the filing (from LOAN_CD.LOAN_AMT2)"
    )

    class Meta:
        """
        Model options.
        """
        app_label = 'calaccess_processed_filings'
        abstract = True


class Form460ScheduleH2ItemOld(Form460ScheduleH2ItemBaseOld):
    """
    Repayments on loans/forgiven loans made by campaign filers circa 2001.

    These transactions are itemized on Schedule H, Part 2, of the most recent
    version to each Form 460 filing in the pre-2001 format. For loan repayments
    and forgiven loans on any version of any Form 460 filing in the pre-2001
    format, see Form460ScheduleH2ItemVersionOld.

    Derived from LOAN_CD records where FORM_TYPE is 'H2'.
    """
    filing = models.ForeignKey(
        'Form460Filing',
        related_name='schedule_h2_items_old',
        null=True,
        on_delete=models.SET_NULL,
        help_text='Foreign key referring to the Form 460 on which the loan '
                  'was reported (from LOAN_CD.FILING_ID)',
    )

    class Meta:
        """
        Model options.
        """
        app_label = 'calaccess_processed_filings'
        unique_together = ((
            'filing',
            'line_item',
        ),)
        verbose_name = "Form 460 (Campaign Disclosure) Schedule H2 old item"

    def __str__(self):
        return '%s-%s' % (self.filing, self.line_item)


class Form460ScheduleH2ItemVersionOld(Form460ScheduleH2ItemBaseOld):
    """
    Every version of each repayment/forgiveness of a loan by a campaign filer circa 2001.

    For loan repayments and forgiven loans on any version of any Form 460 filing
    in the pre-2001 format, see Form460ScheduleH2ItemOld.

    Derived from LOAN_CD records where FORM_TYPE is 'H2'.
    """
    filing_version = models.ForeignKey(
        'Form460FilingVersion',
        related_name='schedule_h2_items_old',
        null=True,
        on_delete=models.SET_NULL,
        help_text='Foreign key referring to the version of the Form 460 that '
                  'includes the outstanding loan'
    )

    class Meta:
        """
        Model options.
        """
        app_label = 'calaccess_processed_filings'
        unique_together = ((
            'filing_version',
            'line_item',
        ),)
        index_together = ((
            'filing_version',
            'line_item',
        ),)
        verbose_name = "Form 460 (Campaign Disclosure) Schedule H2 old item version"

    def __str__(self):
        return '%s-%s-%s' % (
            self.filing_version.filing_id,
            self.filing_version.amend_id,
            self.line_item
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Proxy models for augmenting our source data tables with methods useful for processing.
"""
from .base import ScrapedElectionProxyMixin, ScrapedNameMixin
from .candidates import ScrapedCandidateProxy
from .candidateelections import ScrapedCandidateElectionProxy
from .incumbents import ScrapedIncumbentProxy
from .incumbentelections import ScrapedIncumbentElectionProxy
from .propositions import ScrapedPropositionProxy
from .propositionelections import ScrapedPropositionElectionProxy


__all__ = (
    'ScrapedElectionProxyMixin',
    'ScrapedNameMixin',
    'ScrapedCandidateProxy',
    'ScrapedCandidateElectionProxy',
    'ScrapedIncumbentProxy',
    'ScrapedIncumbentElectionProxy',
    'ScrapedPropositionProxy',
    'ScrapedPropositionElectionProxy',
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Import all of the managers from submodules and thread them together.
"""
from .candidacies import (
    OCDCandidacyQuerySet,
    OCDCandidacyManager
)
from .candidatecontests import (
    OCDCandidateContestQuerySet,
    OCDCandidateContestManager
)
from .elections import (
    OCDPartisanPrimaryManager,
    OCDElectionManager
)
from .parties import OCDPartyManager


__all__ = (
    "OCDCandidacyQuerySet",
    "OCDCandidacyManager",
    "OCDCandidateContestQuerySet",
    "OCDCandidateContestManager",
    "OCDPartisanPrimaryManager",
    "OCDElectionManager",
    "OCDPartyManager"
)



from __future__ import unicode_literals
from django.db import migrations
from calaccess_processed.postgres import CryptoExtension


class Migration(migrations.Migration):

    dependencies = [
        ('calaccess_processed', '0001_squashed_0014_rawfilertofilertypecdproxy_scrapedcandidateelectionproxy_scrapedcandidateproxy_scrapedincumbentelect'),
    ]

    operations = [CryptoExtension(),]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Load OCD Organization model with parties extracted from raw CAL-ACCESS data.
"""
import re
from opencivicdata.core.models import Organization
from calaccess_raw.models.common import LookupCodesCd
from calaccess_processed.management.commands import CalAccessCommand


class Command(CalAccessCommand):
    """
    Load OCD Organization model with parties extracted from raw CAL-ACCESS data.
    """
    help = 'Load OCD Organization model with parties extracted from raw CAL-ACCESS data'
    first_letters_regex = re.compile(r'([A-z])\w+')

    def handle(self, *args, **options):
        """
        Make it happen.
        """
        super(Command, self).handle(*args, **options)
        self.header('Loading Parties')
        self.load()
        self.success("Done!")

    def load(self):
        """
        Insert Party records from the raw LOOKUP_CODES_CD table.
        """
        # Pull all of the raw LookupCodes in the 16000 code_type series
        # exclude the title entry of "PARTY CODE" with the identical number
        object_list = LookupCodesCd.objects.filter(code_type=16000).exclude(code_id=16000)

        # Loop through them all
        for obj in object_list:
            # Pull out the party name ...
            # ... but treat INDEPENDENT and NON-PARTISAN as NO PARTY PREFERENCE
            if obj.code_desc in ['INDEPENDENT', 'NON-PARTISAN', 'N/A', 'NON PARTISAN']:
                party_name = 'NO PARTY PREFERENCE'
            else:
                party_name = obj.code_desc

            # Get a party object from the OCD Organization model
            party, created = Organization.objects.get_or_create(name=party_name, classification='party')

            # If it's new...
            if created:
                # Log it out
                if self.verbosity > 2:
                    self.log(" Created %s party" % party)
                # Save abbreviation as an alternative party name
                # combine the first char of each word (except AND)
                trimmed_party = party_name.upper().replace(' AND ', '')
                first_letters = self.first_letters_regex.findall(trimmed_party)
                abbreviation = ''.join(first_letters)
                party.other_names.get_or_create(name=abbreviation, note='abbreviation')

            # keep the code_id too
            p_id, created = party.identifiers.get_or_create(
                scheme='calaccess_lookup_code_id',
                identifier=obj.code_id,
            )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Abstract base models for Form 497 models.
"""
from .base import Form497ItemBase
from .filing import Form497Filing, Form497FilingVersion
from .part1 import Form497Part1ItemBase, Form497Part1Item, Form497Part1ItemVersion
from .part2 import Form497Part2ItemBase, Form497Part2Item, Form497Part2ItemVersion


__all__ = (
    "Form497Filing",
    "Form497FilingVersion",
    "Form497ItemBase",
    "Form497Part1ItemBase",
    "Form497Part1Item",
    "Form497Part1ItemVersion",
    "Form497Part2ItemBase",
    "Form497Part2Item",
    "Form497Part2ItemVersion"
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Models for storing data from Schedule 497, the Late Contribution Reports.

More about the filing: http://calaccess.californiacivicdata.org/documentation/calaccess-forms/f497/
"""
# Models
from django.db import models
from calaccess_processed_filings.models.campaign import CampaignFinanceFilingBase


class Form497Filing(CampaignFinanceFilingBase):
    """
    The most recent version of each Schedule 497 filing by campaign filers.

    Form 497 is the Late Contribution Report filed by state and local committees
    making or receiving contributions totaling $1,000 or more in the 90 days
    before an election, committees reporting contributions of $5,000 or more in
    connection with a state ballot measure and state candidates as well as state
    ballot measure committees that receive $5,000 or more at any time other than
    a 90-day election cycle.

    Includes information from the cover sheet of the most recent amendment to
    each filing. All versions of Schedule 497 filings can be found in
    schedule497version.
    """
    filing_id = models.IntegerField(
        verbose_name='filing id',
        primary_key=True,
        null=False,
        help_text='Unique identification number for the Schedule 497 filing ('
                  'from CVR_CAMPAIGN_DISCLOSURE_CD.FILING_ID)',
    )
    amendment_count = models.IntegerField(
        verbose_name='Count amendments',
        null=False,
        help_text='Number of amendments to the Schedule 497 filing (from '
                  'maximum value of CVR_CAMPAIGN_DISCLOSURE_CD.AMEND_ID)',
    )

    class Meta:
        """
        Model options.
        """
        app_label = 'calaccess_processed_filings'
        index_together = ((
            'filing_id',
            'amendment_count',
        ),)
        verbose_name = "Form 497 (Late Contribution) filing"

    def __str__(self):
        return str(self.filing_id)


class Form497FilingVersion(CampaignFinanceFilingBase):
    """
    Every version of each Schedule 497 filing by a campaign filer.

    Includes information found on the cover sheet of each version of each
    Schedule 497 filing. For the most recent version of each filing, see
    schedule497filing.
    """
    filing = models.ForeignKey(
        'Form497Filing',
        related_name='versions',
        db_constraint=False,
        null=True,
        on_delete=models.SET_NULL,
        help_text='Unique identification number for the Schedule 497 filing ('
                  'from S497_CD.FILING_ID)',
    )
    amend_id = models.IntegerField(
        null=False,
        help_text='Identifies the version of the Schedule 497 filing, with 0 '
                  'representing the initial filing (from CVR_CAMPAIGN_'
                  'DISCLOSURE_CD.AMEND_ID)',
    )

    class Meta:
        """
        Model options.
        """
        app_label = 'calaccess_processed_filings'
        unique_together = ((
            'filing',
            'amend_id',
        ),)
        index_together = ((
            'filing',
            'amend_id',
        ),)
        verbose_name = "Form 497 (Late Contribution) filing version"

    def __str__(self):
        return '%s-%s' % (self.filing, self.amend_id)



# from toolbox import views
from django.conf import settings
from django.contrib import admin
from django.views.static import serve
from django.conf.urls import include, url
admin.autodiscover()


urlpatterns = (
    # url(r'^$', views.index, name="index"),
    # url(r'^elections/$', views.election_list, name="election_list"),
    # url(r'^elections/(?P<id>(.*))/$', views.election_detail, name="election_detail"),
    # url(r'^candidatecontest/(?P<pk>(.*))/$', views.CandidateContestDetail.as_view(), name="candidatecontest_detail"),
    # url(r'^posts/(?P<pk>(.*))/$', views.PostDetail.as_view(), name="post_detail"),
    # url(r'^people/$', views.PersonList.as_view(), name="person_list"),
    # url(r'^people/(?P<pk>(.*))/$', views.PersonDetail.as_view(), name="person_detail"),
    # url(r'^candidates-no-party/$', views.CandidateNoPartyList.as_view(), name="candidatenoparty_list"),
    # url(r'^primaries-no-party/$', views.PrimaryNoPartyList.as_view(), name="primarynoparty_list"),
    # url(r'^parties/$', views.PartyList.as_view(), name="party_list"),
    # url(r'^parties/(?P<pk>(.*))/$', views.PartyDetail.as_view(), name="party_detail"),
    # url(r'^committees/$', views.CommitteeList.as_view(), name="committee_list"),
    # url(r'^committees/(?P<pk>(.*))/$', views.CommitteeDetail.as_view(), name="committee_detail"),
    # url(r'^filings/(?P<pk>(.*))/$', views.FilingDetail.as_view(), name="filing_detail"),
    # url(r'^filing-actions/(?P<pk>(.*))/$', views.FilingActionDetail.as_view(), name="filingaction_detail"),
    url(r'^admin/', admin.site.urls),
    # url(r'^static/(?P<path>.*)$', serve, {
    #     'document_root': settings.STATIC_ROOT,
    #     'show_indexes': True,
    # }),
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Import all of the managers from submodules and thread them together.
"""
from .ballotmeasurecontests import OCDFlatBallotMeasureContestManager
from .candidacies import OCDFlatCandidacyManager
from .retentioncontests import OCDFlatRetentionContestManager


__all__ = (
    "OCDFlatBallotMeasureContestManager",
    "OCDFlatCandidacyManager",
    "OCDFlatRetentionContestManager",
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Flush data from OCD models.
"""
from opencivicdata.core.models import (
    Jurisdiction,
    Membership,
    Organization,
    Person,
    Post
)
from opencivicdata.elections.models import (
    Candidacy,
    CandidateContest,
    Election,
    BallotMeasureContest,
    RetentionContest
)
from calaccess_processed.management.commands import CalAccessCommand


class Command(CalAccessCommand):
    """
    Flush data from OCD models.
    """
    help = "Flush data from OCD models."

    def handle(self, *args, **options):
        """
        Make it happen.
        """
        super(Command, self).handle(*args, **options)

        # Flush models
        qs_list = [
            Candidacy.objects.all(),
            CandidateContest.objects.all(),
            BallotMeasureContest.objects.all(),
            RetentionContest.objects.all(),
            Election.objects.all(),
            Membership.objects.all(),
            Person.objects.all(),
            Post.objects.all(),
            Organization.objects.all(),
            Jurisdiction.objects.all(),
        ]
        for qs in qs_list:
            if self.verbosity > 0:
                self.log("Flushing {} {} objects".format(qs.count(), qs.model.__name__))
            qs.delete()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Models for storing data from Campaign Disclosure Statements (Form 460).
"""
from django.db import models
from calaccess_processed_filings.models.base import FilingBaseModel
from calaccess_processed_filings.models.campaign import CampaignContributionBase


#
# Summaries
#

class Form460ScheduleCSummaryBase(FilingBaseModel):
    """
    Abstract base model with summary data from Schedule C attachments to Form 460 filings.

    Includes totals for itemized versus unitemized non-monetary contributions.
    """
    itemized_contributions = models.FloatField(
        verbose_name='itemized contributions',
        null=True,
        help_text="Amount received this period - itemized nonmonetary contributions. \
(Include all Schedule C subtotals)",
    )
    unitemized_contributions = models.FloatField(
        verbose_name='unitemized contributions',
        null=True,
        help_text="Amount received this period - unitemized nonmonetary contributions of less than $100",
    )
    total_contributions = models.FloatField(
        verbose_name='total contributions',
        null=True,
        help_text="Total nonmonetary contributions received this period. (Add Lines 1 and 2. Enter here and on the \
Summary Page, Column A, Lines 4 and 10.)",
    )

    class Meta:
        """
        Model options.
        """
        app_label = 'calaccess_processed_filings'
        abstract = True


class Form460ScheduleCSummary(Form460ScheduleCSummaryBase):
    """
    The summary data included with the most recent version a Schedule C attachment to a Form 460 filing.

    Includes totals for itemized versus unitemized non-monetary contributions.
    """
    filing = models.ForeignKey(
        'Form460Filing',
        related_name='schedule_c_summaries',
        null=True,
        on_delete=models.SET_NULL,
        help_text='Foreign key referring to the Form 460 on which the summary was reported',
    )

    class Meta:
        """
        Model options.
        """
        app_label = 'calaccess_processed_filings'
        verbose_name = "Form 460 (Campaign Disclosure) Schedule C summary"
        verbose_name_plural = "Form 460 (Campaign Disclosure) Schedule C summaries"

    def __str__(self):
        return '%s' % (self.filing)


class Form460ScheduleCSummaryVersion(Form460ScheduleCSummaryBase):
    """
    The summary data included with each version of a Schedule C attachment to a Form 460 filing.

    Includes totals for itemized versus unitemized non-monetary contributions.
    """
    filing_version = models.ForeignKey(
        'Form460FilingVersion',
        related_name='schedule_c_summaries',
        null=True,
        on_delete=models.SET_NULL,
        help_text='Foreign key referring to the version of the Form 460 that '
                  'includes the summary'
    )

    class Meta:
        """
        Model options.
        """
        app_label = 'calaccess_processed_filings'
        verbose_name = "Form 460 (Campaign Disclosure) Schedule C summary version"

    def __str__(self):
        return '%s-%s' % (
            self.filing_version.filing_id,
            self.filing_version.amend_id,
        )


#
# Items
#

class Form460ScheduleCItemBase(CampaignContributionBase):
    """
    Abstract base model for items reported on Schedule C of Form 460 filings.

    On Schedule C, campaign filers are required to itemize nonmonetary
    contributions received during the period covered by the filing.
    """
    fair_market_value = models.DecimalField(
        verbose_name='fair market value',
        decimal_places=2,
        max_digits=14,
        help_text="Amount it would cost to purchase the donated goods or "
                  "services on the open market (from RCPT_CD.AMOUNT)"
    )
    contribution_description = models.CharField(
        max_length=90,
        blank=True,
        help_text="Description of the contributed goods or services (from "
                  "RCPT_CD.CTRIB_DSCR)"
    )

    class Meta:
        """
        Model options.
        """
        app_label = 'calaccess_processed_filings'
        abstract = True


class Form460ScheduleCItem(Form460ScheduleCItemBase):
    """
    Nonmonetary contributions received by campaign filers.

    These transactions are itemized on Schedule C of the most recent version
    of each Form 460 filing. For nonmonetary contributions itemized on any
    version of any Form 460 filing, see Form460ScheduleCItemVersion.

    Derived from RCPT_CD records where FORM_TYPE is 'C'.
    """
    filing = models.ForeignKey(
        'Form460Filing',
        related_name='schedule_c_items',
        null=True,
        on_delete=models.SET_NULL,
        help_text='Foreign key referring to the Form 460 on which the monetary'
                  ' contribution was reported (from RCPT_CD.FILING_ID)',
    )

    class Meta:
        """
        Model options.
        """
        app_label = 'calaccess_processed_filings'
        unique_together = ((
            'filing',
            'line_item',
        ),)
        verbose_name = "Form 460 (Campaign Disclosure) Schedule C item"

    def __str__(self):
        return '%s-%s' % (self.filing, self.line_item)


class Form460ScheduleCItemVersion(Form460ScheduleCItemBase):
    """
    Every version of each nonmonetary contribution received by a campaign filer.

    For nonmonetary contributions itemized on Schedule C of the most recent
    version of each Form 460 filing, see Form460ScheduleCItem.

    Derived from RCPT_CD records where FORM_TYPE is 'C'.
    """
    filing_version = models.ForeignKey(
        'Form460FilingVersion',
        related_name='schedule_c_items',
        null=True,
        on_delete=models.SET_NULL,
        help_text='Foreign key referring to the version of the Form 460 that '
                  'includes the received contribution'
    )

    class Meta:
        """
        Model options.
        """
        app_label = 'calaccess_processed_filings'
        unique_together = ((
            'filing_version',
            'line_item',
        ),)
        index_together = ((
            'filing_version',
            'line_item',
        ),)
        verbose_name = "Form 460 (Campaign Disclosure) Schedule C item version"

    def __str__(self):
        return '%s-%s-%s' % (
            self.filing_version.filing_id,
            self.filing_version.amend_id,
            self.line_item
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
General utilities for the application.
"""
import logging
from datetime import date
default_app_config = 'calaccess_processed_elections.apps.CalAccessProcessedElectionsConfig'
logger = logging.getLogger(__name__)


def get_expected_election_date(year, election_type):
    """
    Get the date of the election in the given year and type.

    Raise an exception if year is not even or if election_type is not
    "PRIMARY" or "GENERAL".

    Return a date object.
    """
    # Rules defined here:
    # https://leginfo.legislature.ca.gov/faces/codes_displayText.xhtml?lawCode=ELEC&division=1.&title=&part=&chapter=1.&article= # noqa
    logger.debug("Getting expected election date for {} {}".format(year, election_type))
    if year % 2 != 0:
        raise ValueError("Regular elections occur in even years.")
    elif election_type.upper() == 'PRIMARY':
        # Primary elections are in June
        month = 6
    elif election_type.upper() == 'GENERAL':
        # General elections are in November
        month = 11
    else:
        raise ValueError("election_type must 'PRIMARY' or 'GENERAL'.")

    # get the first weekday
    # zero-indexed starting with monday
    first_weekday = date(year, month, 1).weekday()

    # calculate day or first tuesday after first monday
    day_or_month = (7 - first_weekday) % 7 + 2

    return date(year, month, day_or_month)


__all__ = (
    "get_expected_election_date"
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Proxy models for augmenting our source data tables with methods useful for processing.
"""
import datetime
from django.db import transaction
from django.db.models import Count
from .proxies import OCDCandidacyProxy, OCDPersonProxy


def merge_persons(persons):
    """
    Merge items in persons iterable into one Person object, which is returned.
    """
    # each person will be merged into this one
    keep = persons.pop(0)
    if keep.__class__ != OCDPersonProxy:
        keep.__class__ = OCDPersonProxy

    # loop over all the rest
    for i in persons:
        if i.__class__ != OCDPersonProxy:
            i.__class__ = OCDPersonProxy
        merge(keep, i)
        keep.refresh_from_db()

    dedupe_person_ids(keep)
    dedupe_person_candidacies(keep)
    keep.refresh_from_db()

    # make sure Person name is same as most recent candidate_name
    latest_candidate_name = keep.candidacies.latest(
        'contest__election__date',
    ).candidate_name
    if keep.name != latest_candidate_name:
        # move current Person.name into other_names
        keep.add_other_name(keep.name, 'Updated current name in merge')
        keep.name = latest_candidate_name
    keep.save()

    return keep


def dedupe_person_ids(person):
    """
    Remove duplicate PersonIdentifier objects linked to person.
    """
    filer_ids = person.identifiers.filter(scheme='calaccess_filer_id')

    dupe_filer_ids = filer_ids.values("identifier").annotate(
        row_count=Count('id'),
    ).order_by().filter(row_count__gt=1)

    for i in dupe_filer_ids.all():
        # delete all rows with that filer_id
        person.identifiers.filter(identifier=i['identifier']).delete()
        # then re-add the one
        person.identifiers.create(
            scheme='calaccess_filer_id',
            identifier=i['identifier'],
        )

    return person


def dedupe_person_candidacies(person):
    """
    Remove duplicate Candidacy objects linked to person.
    """
    # first, make groups by contests with more than one candidacy
    contest_group_q = person.candidacies.values("contest").annotate(
        row_count=Count('id')
    ).filter(row_count__gt=1)

    # loop over each contest group
    for group in contest_group_q.all():
        cands = person.candidacies.filter(contest=group['contest'])
        # preference to "qualified" candidacy (from scrape)
        if cands.filter(registration_status='qualified').exists():
            cand_to_keep = cands.filter(registration_status='qualified').all()[0]
        # or the one with the most recent filed_date
        else:
            cand_to_keep = cands.latest('filed_date')

        cand_to_keep.__class__ = OCDCandidacyProxy

        # loop over all the other candidacies in the group
        for cand_to_discard in cands.exclude(id=cand_to_keep.id).all():
            # assuming there's nothing else to preserve in extras
            # besides form501_filing_ids
            if 'form501_filing_ids' in cand_to_discard.extras:
                for i in cand_to_discard.extras['form501_filing_ids']:
                    cand_to_keep.link_form501(i)
            cand_to_keep.refresh_from_db()

            if 'form501_filing_ids' in cand_to_keep.extras:
                cand_to_keep.update_from_form501()
            cand_to_keep.refresh_from_db()

            # keep the candidate_name, if not already somewhere else
            if (
                cand_to_discard.candidate_name != cand_to_keep.candidate_name
                and cand_to_discard.candidate_name != cand_to_keep.person.name
                and not cand_to_keep.person.other_names.filter(
                    name=cand_to_discard.candidate_name
                ).exists()
            ):
                person.other_names.create(
                    name=cand_to_discard.candidate_name,
                    note='From merge of %s candidacies' % cand_to_keep.contest
                )
                cand_to_keep.refresh_from_db()

            # keep the candidacy sources
            if cand_to_discard.sources.exists():
                for source in cand_to_discard.sources.all():
                    if not cand_to_keep.sources.filter(url=source.url).exists():
                        cand_to_keep.sources.create(
                            url=source.url,
                            note=source.note,
                        )
                    cand_to_keep.refresh_from_db()

            # keep earliest filed_date
            if cand_to_keep.filed_date and cand_to_discard.filed_date:
                if cand_to_keep.filed_date > cand_to_discard.filed_date:
                    cand_to_keep.filed_date = cand_to_discard.filed_date
            elif cand_to_discard.filed_date:
                cand_to_keep.filed_date = cand_to_discard.filed_date
            # keep is_incumbent if True
            if not cand_to_keep.is_incumbent and cand_to_discard.is_incumbent:
                cand_to_keep.is_incumbent = cand_to_discard.is_incumbent
            # assuming not trying to merge candidacies with different parties
            if not cand_to_keep.party and cand_to_discard.party:
                cand_to_keep.party = cand_to_discard.party

            cand_to_keep.save()
            cand_to_discard.delete()

    return person


def compute_diff(obj1, obj2):
    """
        Given two objects compute a list of differences.

        Each diff dict has the following keys:
            field - name of the field
            new - the new value for the field
            one - value of the field in obj1
            two - value of the field in obj2
            diff - none|one|two|new
            list - true if field is a list of related objects
    """
    comparison = []
    fields = obj1._meta.get_fields()
    exclude = ('created_at', 'updated_at', 'id', 'locked_fields')

    if obj1 == obj2:
        raise ValueError('cannot merge object with itself')

    for field in fields:
        if field.name in exclude:
            continue
        elif not field.is_relation:
            piece_one = getattr(obj1, field.name)
            piece_two = getattr(obj2, field.name)
            if piece_one == piece_two:
                diff = 'none'
                new = piece_one
            elif piece_one:
                diff = 'one'
                new = piece_one
            elif piece_two:
                diff = 'two'
                new = piece_two
            comparison.append({
                'field': field.name,
                'new': new,
                'one': getattr(obj1, field.name),
                'two': getattr(obj2, field.name),
                'diff': diff,
                'list': False,
            })
        else:
            related_name = field.get_accessor_name()
            piece_one = list(getattr(obj1, related_name).all())
            piece_two = list(getattr(obj2, related_name).all())
            # TODO: try and deduplicate the lists?
            new = piece_one + piece_two
            diff = 'none' if piece_one == piece_two else 'one'

            if (field.name == 'other_names' and obj1.name != obj2.name):
                new.append(field.related_model(name=obj2.name,
                                               note='from merge w/ ' + obj2.id)
                           )
                diff = 'new'
            if field.name == 'identifiers':
                new.append(field.related_model(identifier=obj2.id))
                diff = 'new'
            if field.name == 'memberships':
                new = _dedupe_memberships(new)

            comparison.append({
                'field': related_name,
                'new': new,
                'one': piece_one,
                'two': piece_two,
                'diff': diff,
                'list': True,
            })

    comparison.append({'field': 'created_at',
                       'new': min(obj1.created_at, obj2.created_at),
                       'one': obj1.created_at,
                       'two': obj2.created_at,
                       'diff': 'one' if obj1.created_at < obj2.created_at else 'two',
                       'list': False,
                       })
    comparison.append({'field': 'updated_at',
                       'new': datetime.datetime.utcnow(),
                       'one': obj1.updated_at,
                       'two': obj2.updated_at,
                       'diff': 'new',
                       'list': False,
                       })
    # locked fields are any fields that change that aren't M2M relations
    # (ending in _set)
    new_locked_fields = obj1.locked_fields + obj2.locked_fields + [
        c['field'] for c in comparison if c['diff'] != 'none' and not c['field'].endswith('_set')
    ]
    new_locked_fields = set(new_locked_fields) - {'updated_at', 'created_at'}
    comparison.append({'field': 'locked_fields',
                       'new': list(new_locked_fields),
                       'one': obj1.locked_fields,
                       'two': obj2.updated_at,
                       'diff': 'new',
                       'list': False,
                       })

    return comparison


@transaction.atomic
def apply_diff(obj1, obj2, diff):
    for row in diff:
        if row['diff'] != 'none':
            if row['list']:
                # save items, the ids have been set to obj1
                for item in row['new']:
                    setattr(item,
                            getattr(obj1, row['field']).field.name,
                            obj1)
                    item.save()
            else:
                setattr(obj1, row['field'], row['new'])

    obj1.save()
    count, delete_plan = obj2.delete()
    if count > 1:
        # shouldn't happen, but let's be sure
        raise AssertionError('deletion failed due to related objects left unmerged')


def merge(obj1, obj2):
    diff = compute_diff(obj1, obj2)
    apply_diff(obj1, obj2, diff)


def _dedupe_memberships(memberships):
    deduped = []
    mset = set()
    for membership in memberships:
        mkey = (membership.organization_id,
                membership.label,
                membership.end_date,
                membership.post_id)
        if mkey not in mset:
            deduped.append(membership)
            mset.add(mkey)
        else:
            membership.delete()
    return deduped



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Proxy models for augmenting our source data tables with methods useful for processing.
"""
from .opencivicdata import (
    OCDBallotMeasureContestProxy,
    OCDBallotMeasureContestIdentifierProxy,
    OCDBallotMeasureContestOptionProxy,
    OCDBallotMeasureContestSourceProxy,
    OCDCandidateContestProxy,
    OCDCandidateContestPostProxy,
    OCDCandidateContestSourceProxy,
    OCDCandidacyProxy,
    OCDCandidacySourceProxy,
    OCDElectionProxy,
    OCDElectionIdentifierProxy,
    OCDElectionSourceProxy,
    OCDPartyProxy,
    OCDRetentionContestProxy,
    OCDRetentionContestIdentifierProxy,
    OCDRetentionContestOptionProxy,
    OCDRetentionContestSourceProxy,
    OCDDivisionProxy,
    OCDMembershipProxy,
    OCDJurisdictionProxy,
    OCDOrganizationProxy,
    OCDOrganizationIdentifierProxy,
    OCDOrganizationNameProxy,
    OCDPersonProxy,
    OCDPersonIdentifierProxy,
    OCDPersonNameProxy,
    OCDPostProxy
)
from .calaccess_raw import RawFilerToFilerTypeCdProxy
from .calaccess_scraped import (
    ScrapedElectionProxyMixin,
    ScrapedNameMixin,
    ScrapedCandidateProxy,
    ScrapedIncumbentProxy,
    ScrapedCandidateElectionProxy,
    ScrapedIncumbentElectionProxy,
    ScrapedPropositionProxy,
    ScrapedPropositionElectionProxy
)


__all__ = (
    "OCDBallotMeasureContestProxy",
    "OCDBallotMeasureContestIdentifierProxy",
    "OCDBallotMeasureContestOptionProxy",
    "OCDBallotMeasureContestSourceProxy",
    "OCDCandidateContestProxy",
    "OCDCandidateContestPostProxy",
    "OCDCandidateContestSourceProxy",
    "OCDCandidacyProxy",
    "OCDCandidacySourceProxy",
    "OCDElectionProxy",
    "OCDElectionIdentifierProxy",
    "OCDElectionSourceProxy",
    "OCDPartyProxy",
    "OCDRetentionContestProxy",
    "OCDRetentionContestIdentifierProxy",
    "OCDRetentionContestOptionProxy",
    "OCDRetentionContestSourceProxy",
    'OCDDivisionProxy',
    'OCDJurisdictionProxy',
    'OCDMembershipProxy',
    'OCDOrganizationProxy',
    'OCDOrganizationIdentifierProxy',
    'OCDOrganizationNameProxy',
    'OCDPersonProxy',
    'OCDPersonIdentifierProxy',
    'OCDPersonNameProxy',
    'OCDPostProxy',
    "RawFilerToFilerTypeCdProxy",
    "ScrapedElectionProxyMixin",
    "ScrapedNameMixin",
    'ScrapedCandidateProxy',
    'ScrapedCandidateElectionProxy',
    'ScrapedIncumbentProxy',
    'ScrapedIncumbentElectionProxy',
    'ScrapedPropositionProxy',
    'ScrapedPropositionElectionProxy',
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Proxy models for augmenting our source data tables with methods useful for processing.
"""
from .core import (
    OCDDivisionProxy,
    OCDMembershipProxy,
    OCDOrganizationProxy,
    OCDOrganizationIdentifierProxy,
    OCDOrganizationNameProxy,
    OCDJurisdictionProxy,
    OCDPersonProxy,
    OCDPersonIdentifierProxy,
    OCDPersonNameProxy,
    OCDPostProxy
)
from .elections import (
    OCDBallotMeasureContestProxy,
    OCDBallotMeasureContestIdentifierProxy,
    OCDBallotMeasureContestOptionProxy,
    OCDBallotMeasureContestSourceProxy,
    OCDCandidateContestProxy,
    OCDCandidateContestPostProxy,
    OCDCandidateContestSourceProxy,
    OCDCandidacyProxy,
    OCDCandidacySourceProxy,
    OCDElectionProxy,
    OCDElectionIdentifierProxy,
    OCDElectionSourceProxy,
    OCDPartyProxy,
    OCDRetentionContestProxy,
    OCDRetentionContestIdentifierProxy,
    OCDRetentionContestOptionProxy,
    OCDRetentionContestSourceProxy,
)

__all__ = (
    'RawFilerToFilerTypeCdManager',
    'OCDDivisionProxy',
    'OCDJurisdictionProxy',
    'OCDMembershipProxy',
    'OCDOrganizationProxy',
    'OCDOrganizationIdentifierProxy',
    'OCDOrganizationNameProxy',
    'OCDPersonProxy',
    'OCDPersonIdentifierProxy',
    'OCDPersonNameProxy',
    'OCDPostProxy',
    "OCDBallotMeasureContestProxy",
    "OCDBallotMeasureContestIdentifierProxy",
    "OCDBallotMeasureContestOptionProxy",
    "OCDBallotMeasureContestSourceProxy",
    "OCDCandidateContestProxy",
    "OCDCandidateContestPostProxy",
    "OCDCandidateContestSourceProxy",
    "OCDCandidacyProxy",
    "OCDCandidacySourceProxy",
    "OCDElectionProxy",
    "OCDElectionIdentifierProxy",
    "OCDElectionSourceProxy",
    "OCDPartyProxy",
    "OCDRetentionContestProxy",
    "OCDRetentionContestIdentifierProxy",
    "OCDRetentionContestOptionProxy",
    "OCDRetentionContestSourceProxy"
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Models for storing data from Campaign Disclosure Statements (Form 460).
"""
from django.db import models
from calaccess_processed_filings.models.base import FilingBaseModel
from calaccess_processed_filings.models.campaign import (
    CampaignExpenditureItemBase,
    CampaignExpenditureSubItemBase
)

#
# Summaries
#


class Form460ScheduleESummaryBase(FilingBaseModel):
    """
    Abstract base model with summary data from Schedule E attachments to Form 460 filings.

    Includes totals for itemized versus unitemized expenditures.
    """
    itemized_expenditures = models.FloatField(
        verbose_name='itemized expenditures',
        null=True,
        help_text="Payments made this period of $100 or more. (Include all Schedule E subtotals.)",
    )
    unitemized_expenditures = models.FloatField(
        verbose_name='unitemized expenditures',
        null=True,
        help_text="Unitemized payments made this period of under $100.",
    )
    interest_paid = models.FloatField(
        verbose_name='interest paid',
        null=True,
        help_text="Total interest paid this period on loans. (Enter amount from Schedule B, Part 1, Column (e).)",
    )
    total_expenditures = models.FloatField(
        verbose_name='total expenditures',
        null=True,
        help_text="Total payments made this period. (Add lines 1, 2, and 3. Enter here and on the Summary Page, Column \
A, Line 6.) ",
    )

    class Meta:
        """
        Model options.
        """
        app_label = 'calaccess_processed_filings'
        abstract = True


class Form460ScheduleESummary(Form460ScheduleESummaryBase):
    """
    The summary data included with the most recent version a Schedule E attachment to a Form 460 filing.

    Includes totals for itemized versus unitemized expenditures.
    """
    filing = models.ForeignKey(
        'Form460Filing',
        related_name='schedule_e_summaries',
        null=True,
        on_delete=models.SET_NULL,
        help_text='Foreign key referring to the Form 460 on which the summary was reported',
    )

    class Meta:
        """
        Model options.
        """
        app_label = 'calaccess_processed_filings'
        verbose_name = "Form 460 (Campaign Disclosure) Schedule E summary"
        verbose_name_plural = "Form 460 (Campaign Disclosure) Schedule E summaries"

    def __str__(self):
        return '%s' % (self.filing)


class Form460ScheduleESummaryVersion(Form460ScheduleESummaryBase):
    """
    The summary data included with each version of a Schedule E attachment to a Form 460 filing.

    Includes totals for itemized versus unitemized expenditures.
    """
    filing_version = models.ForeignKey(
        'Form460FilingVersion',
        related_name='schedule_e_summaries',
        null=True,
        on_delete=models.SET_NULL,
        help_text='Foreign key referring to the version of the Form 460 that '
                  'includes the summary'
    )

    class Meta:
        """
        Model options.
        """
        app_label = 'calaccess_processed_filings'
        verbose_name = "Form 460 (Campaign Disclosure) Schedule E summary version"

    def __str__(self):
        return '%s-%s' % (
            self.filing_version.filing_id,
            self.filing_version.amend_id,
        )

#
# Items
#


class Form460ScheduleEItem(CampaignExpenditureItemBase):
    """
    Payments made by campaign filers.

    These transactions are itemized on Schedule E of the most recent version
    of each Form 460 filing. For payments itemized on any version of any
    filing, see Form460ScheduleEItemVersion.

    Does not include:
    * Interest paid on loans received
    * Loans made to others
    * Transfers of campaign funds into savings accounts
    * Payments made by agents or contractors on behalf of the filer
    * Certificates of deposit
    * Money market accounts
    * Purchases of other assets that can readily be converted to cash

    Derived from EXPN_CD records where FORM_TYPE is 'E'.
    """
    filing = models.ForeignKey(
        'Form460Filing',
        related_name='schedule_e_items',
        null=True,
        on_delete=models.SET_NULL,
        help_text='Foreign key referring to the Form 460 on which the '
                  'payment was reported (from EXPN_CD.FILING_ID)',
    )

    class Meta:
        """
        Model options.
        """
        app_label = 'calaccess_processed_filings'
        unique_together = ((
            'filing',
            'line_item',
        ),)
        verbose_name = "Form 460 (Campaign Disclosure) Schedule E item"

    def __str__(self):
        return '%s-%s' % (self.filing, self.line_item)


class Form460ScheduleEItemVersion(CampaignExpenditureItemBase):
    """
    Every version of each payment made by a campaign filer.

    For payments itemized on Schedule E of the most recent version of each Form
    460 filing, see Form460ScheduleEItem.

    Does not include:
    * Interest paid on loans received
    * Loans made to others
    * Transfers of campaign funds into savings accounts
    * Payments made by agents or contractors on behalf of the filer
    * Certificates of deposit
    * Money market accounts
    * Purchases of other assets that can readily be converted to cash

    Derived from EXPN_CD records where FORM_TYPE is 'E'.
    """
    filing_version = models.ForeignKey(
        'Form460FilingVersion',
        related_name='schedule_e_items',
        null=True,
        on_delete=models.SET_NULL,
        help_text='Foreign key referring to the version of the Form 460 that '
                  'includes the payment made'
    )

    class Meta:
        """
        Model options.
        """
        app_label = 'calaccess_processed_filings'
        unique_together = ((
            'filing_version',
            'line_item',
        ),)
        index_together = ((
            'filing_version',
            'line_item',
        ),)
        verbose_name = "Form 460 (Campaign Disclosure) Schedule E item version"

    def __str__(self):
        return '%s-%s-%s' % (
            self.filing_version.filing_id,
            self.filing_version.amend_id,
            self.line_item
        )


class Form460ScheduleESubItem(CampaignExpenditureSubItemBase):
    """
    Sub-items of payments made by campaign filers.

    These transactions are itemized on Schedule E of the most recent version
    of each Form 460 filing. For sub-item payments on any version of any Form
    460 filing, see Form460ScheduleESubItemVersion.

    A sub-item is a transaction where the amount is lumped into another
    "parent" payment reported elsewhere on the filing.

    Includes:
    * Payments supporting or opposing other candidates, ballot measures
    or committees, which are summarized on Schedule D
    * Payments made to vendors over $100 included in credit card payments
    * Payments made by agents or independent contractors on behalf of the
    campaign filer which were reported on Schedule E instead of G
    * Payments made on the accrued expenses reported on Schedule F

    Derived from EXPN_CD records where FORM_TYPE is 'E' and MEMO_CODE is not
    blank.
    """
    filing = models.ForeignKey(
        'Form460Filing',
        related_name='schedule_e_subitems',
        null=True,
        on_delete=models.SET_NULL,
        help_text='Foreign key referring to the Form 460 on which the '
                  'payment was reported (from EXPN_CD.FILING_ID)',
    )

    class Meta:
        """
        Model options.
        """
        app_label = 'calaccess_processed_filings'
        unique_together = ((
            'filing',
            'line_item',
        ),)
        verbose_name = "Form 460 (Campaign Disclosure) Schedule E subitem"

    def __str__(self):
        return '%s-%s' % (self.filing, self.line_item)


class Form460ScheduleESubItemVersion(CampaignExpenditureSubItemBase):
    """
    Every version of each sub-item of a payment by a campaign filer.

    For payments sub-itemized on Schedule E of the most recent version of each
    Form 460 filing, see Form460ScheduleESubItem.

    A sub-item is a transaction where the amount is lumped into another
    "parent" payment reported elsewhere on the filing.

    Includes:
    * Payments supporting or opposing other candidates, ballot measures
    or committees, which are summarized on Schedule D
    * Payments made to vendors over $100 included in credit card payments
    * Payments made by agents or independent contractors on behalf of the
    campaign filer which were reported on Schedule E instead of G
    * Payments made on the accrued expenses reported on Schedule F

    Derived from EXPN_CD records where FORM_TYPE is 'E' and MEMO_CODE is not
    blank.
    """
    filing_version = models.ForeignKey(
        'Form460FilingVersion',
        related_name='schedule_e_subitems',
        null=True,
        on_delete=models.SET_NULL,
        help_text='Foreign key referring to the version of the Form 460 that '
                  'includes the payment made'
    )

    class Meta:
        """
        Model options.
        """
        app_label = 'calaccess_processed_filings'
        unique_together = ((
            'filing_version',
            'line_item',
        ),)
        index_together = ((
            'filing_version',
            'line_item',
        ),)
        verbose_name = "Form 460 (Campaign Disclosure) Schedule E subitem version"

    def __str__(self):
        return '%s-%s-%s' % (
            self.filing_version.filing_id,
            self.filing_version.amend_id,
            self.line_item
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Proxy models for augmenting our source data tables with methods useful for processing.
"""
from __future__ import unicode_literals, absolute_import
from calaccess_raw.models import FilerToFilerTypeCd
from calaccess_processed_elections.managers import RawFilerToFilerTypeCdManager


class RawFilerToFilerTypeCdProxy(FilerToFilerTypeCd):
    """
    Proxy model with extra tools for working with the calaccess_raw FilerToFilerTypeCd model.
    """
    objects = RawFilerToFilerTypeCdManager()

    class Meta:
        """
        Make this a proxy model.
        """
        app_label = "calaccess_processed_elections"
        proxy = True



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Custom administration panels for campaign models.
"""
from __future__ import unicode_literals
from django.contrib import admin
from calaccess_processed_filings import models
from calaccess_raw.admin.base import BaseAdmin


@admin.register(models.Form461Filing)
class Form461FilingAdmin(BaseAdmin):
    """
    Custom admin for the Form461Filing model.
    """
    list_display = (
        'filing_id',
        'amendment_count',
        'filer_id',
        'filer_lastname',
        'filer_firstname',
        'date_filed',
        'election_date',
    )


@admin.register(models.Form461FilingVersion)
class Form461FilingVersionAdmin(BaseAdmin):
    """
    Custom admin for the Form461FilingVersion model.
    """
    list_display = (
        'filing',
        'amend_id',
        'filer_id',
        'filer_lastname',
        'filer_firstname',
        'date_filed',
        'election_date',
    )


@admin.register(models.Form461Part5Item)
class Form461Part5ItemAdmin(BaseAdmin):
    """
    Custom admin for the Form461Part5Item model.
    """
    list_display = (
        'filing',
        'line_item',
        'expense_date',
        'payee_code',
        'payee_lastname',
        'payee_firstname',
        'amount',
        'cumulative_ytd_amount',
        'transaction_id',
    )


@admin.register(models.Form461Part5ItemVersion)
class Form461Part5ItemVersionAdmin(BaseAdmin):
    """
    Custom admin for the Form461Part5ItemVersion model.
    """
    list_display = (
        'filing_version',
        'line_item',
        'expense_date',
        'payee_code',
        'payee_lastname',
        'payee_firstname',
        'amount',
        'cumulative_ytd_amount',
        'transaction_id',
    )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Export sample json for OCDEP.
"""
from __future__ import print_function
from django.core.management.base import BaseCommand
from django.core.serializers.json import DjangoJSONEncoder
from calaccess_processed.models.opencivicdata.elections import Election
from calaccess_processed.models.opencivicdata.elections.ballot_selection import (
    BallotMeasureSelection,
    CandidateSelection,
)
from calaccess_processed.models.opencivicdata.elections.ballot_selection import (
    BallotSelectionBase,
    BallotMeasureSelection,
    CandidateSelection,
)
from calaccess_processed.models.opencivicdata.elections.candidacy import Candidacy
from calaccess_processed.models.opencivicdata.elections.contest import (
    ContestBase,
    BallotMeasureContest,
    CandidateContest,
    RetentionContest,
)
from calaccess_processed.models.opencivicdata.elections.party import Party
import re


class Command(BaseCommand):
    """
    Export sample json for OCDEP.
    """
    help = 'Export sample json for OCDEP.'

    def handle(self, *args, **options):
        """
        Do it.
        """
        self.encoder = DjangoJSONEncoder(indent=4)

        # Election
        elec = Election.objects.latest('start_time')
        full_elec = self.prep_dict(elec.__dict__).copy()
        full_elec.update(self.prep_dict(elec.event_ptr.__dict__))
        full_elec['identifiers'] = [
            self.prep_dict(i.__dict__) for i in elec.identifiers.all()
        ]
        full_elec['sources'] = [
            self.prep_dict(i.__dict__) for i in elec.sources.all()
        ]

        print('Sample Election')
        print('+++++++++++++++\n\n')
        print('.. code:: javascript\n')
        print(self.encoder.encode(full_elec))
        print('--------------------------------------------------------')

        # ContestBase
        contest = ContestBase.objects.get(
            election=elec,
            name='STATE SENATE 01',
        )
        full_contest = self.prep_dict(contest.__dict__).copy()
        full_contest['identifiers'] = [
            self.prep_dict(i.__dict__) for i in contest.identifiers.all()
        ]
        full_contest['sources'] = [
            self.prep_dict(i.__dict__) for i in contest.sources.all()
        ]

        print('Sample ContestBase')
        print('+++++++++++++++++++\n\n')
        print('.. code:: javascript\n')
        print(self.encoder.encode(full_contest))
        print('--------------------------------------------------------')

        # BallotMeasureContest
        ballot_measure = BallotMeasureContest.objects.get(
            election=elec,
            name='PROPOSITION 060- ADULT FILMS. CONDOMS. HEALTH REQUIREMENTS. INITIATIVE STATUTE.',
        )
        full_ballot_measure = self.prep_dict(ballot_measure.__dict__).copy()
        full_ballot_measure['identifiers'] = [
            self.prep_dict(i.__dict__) for i in ballot_measure.identifiers.all()
        ]
        full_ballot_measure['sources'] = [
            self.prep_dict(i.__dict__) for i in ballot_measure.sources.all()
        ]

        print('Sample BallotMeasureContest')
        print('+++++++++++++++++++++++++++\n\n')
        print('.. code:: javascript\n')
        print(self.encoder.encode(full_ballot_measure))
        print('--------------------------------------------------------')

        # CandidateContest
        candidate_contest = CandidateContest.objects.get(
            election=elec,
            name='STATE SENATE 01',
        )
        full_candidate_contest = self.prep_dict(candidate_contest.__dict__).copy()
        full_candidate_contest['identifiers'] = [
            self.prep_dict(i.__dict__) for i in candidate_contest.identifiers.all()
        ]
        full_candidate_contest['sources'] = [
            self.prep_dict(i.__dict__) for i in candidate_contest.sources.all()
        ]

        print('Sample CandidateContest')
        print('+++++++++++++++++++++++\n\n')
        print('.. code:: javascript\n')
        print(self.encoder.encode(full_candidate_contest))
        print('--------------------------------------------------------')

        # RetentionContest
        retention_contest = RetentionContest.objects.get(
            name='2003 RECALL QUESTION',
        )
        full_retention_contest = self.prep_dict(retention_contest.__dict__).copy()
        full_retention_contest['identifiers'] = [
            self.prep_dict(i.__dict__) for i in retention_contest.identifiers.all()
        ]
        full_retention_contest['sources'] = [
            self.prep_dict(i.__dict__) for i in retention_contest.sources.all()
        ]

        print('Sample RetentionContest')
        print('+++++++++++++++++++++++\n\n')
        print('.. code:: javascript\n')
        print(self.encoder.encode(full_retention_contest))
        print('--------------------------------------------------------')

        # Candidacy
        candidacy = CandidateSelection.objects.filter(
            contest__election=elec,
            contest__name='STATE SENATE 01',
        )[0].candidacies.all()[0]
        full_candidacy = self.prep_dict(candidacy.__dict__).copy()
        full_candidacy['sources'] = [
            self.prep_dict(i.__dict__) for i in candidacy.sources.all()
        ]

        print('Sample Candidacy')
        print('++++++++++++++++\n\n')
        print('.. code:: javascript\n')
        print(self.encoder.encode(full_candidacy))
        print('--------------------------------------------------------')

        # Party
        party = Party.objects.get(name='DEMOCRATIC')
        full_party = self.prep_dict(party.__dict__).copy()
        full_party['sources'] = [
            self.prep_dict(i.__dict__) for i in party.sources.all()
        ]

        print('Sample Party')
        print('++++++++++++++++\n\n')
        print('.. code:: javascript\n')
        print(self.encoder.encode(full_party))
        print('--------------------------------------------------------')

        # BallotSelectionBase
        ballot_selection = BallotSelectionBase.objects.filter(
            candidateselection__contest__election=elec,
            candidateselection__contest__name='STATE SENATE 01',
        )[0]
        full_ballot_selection = self.prep_dict(
            ballot_selection.__dict__
        ).copy()

        print('Sample BallotSelectionBase')
        print('++++++++++++++++++++++++++\n\n')
        print('.. code:: javascript\n')
        print(self.encoder.encode(full_ballot_selection))
        print('--------------------------------------------------------')

        # BallotMeasureSelection
        ballot_measure_selection = BallotMeasureSelection.objects.get(
            contest__name='PROPOSITION 060- ADULT FILMS. CONDOMS. HEALTH REQUIREMENTS. INITIATIVE STATUTE.',
            selection='Yes',
        )
        full_ballot_measure_selection = self.prep_dict(
            ballot_measure_selection.__dict__
        ).copy()

        print('Sample BallotMeasureSelection')
        print('++++++++++++++++++++\n\n')
        print('.. code:: javascript\n')
        print(self.encoder.encode(full_ballot_measure_selection))
        print('--------------------------------------------------------')

        # CandidateSelection
        candidate_selection = CandidateSelection.objects.filter(
            contest__election=elec,
            contest__name='STATE SENATE 01',
        )[0]
        full_candidate_selection = self.prep_dict(
            candidate_selection.__dict__
        ).copy()
        full_candidate_selection['candidacy_ids'] = [
            i.id for i in candidate_selection.candidacies.all()
        ]

        print('Sample CandidateSelection')
        print('++++++++++++++++++++++++++\n\n')
        print('.. code:: javascript\n')
        print(self.encoder.encode(full_candidate_selection))
        print('--------------------------------------------------------')

    def prep_dict(self, obj_dict):
        """
        Remove unwanted items from given object.
        """
        cp_dict = obj_dict.copy()
        for k in obj_dict:
            if re.match(r'^_.+$', k) or '_ptr_' in k:
                del cp_dict[k]

        if 'scheme' in cp_dict or 'url' in cp_dict:
            _id = [k for k in cp_dict if k.endswith('_id')][0]
            del cp_dict[_id]
            del cp_dict['id']

        return cp_dict
